package org.eso.oca.data;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;


/**
 * Class CalibColumnsDAO
 * 
 * Handles database calls for the calib_columns table.
 *
 *
 * @version $Revision: 1.6 $
 * @author pnunes, Mar 30, 2005
 */


public class CalibColumnsDAO {

    /**
     * Column tname of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TNAME = 0;
    public static final int TYPE_TNAME = Types.VARCHAR;
    public static final String NAME_TNAME = "tname";

    /**
     * Column name of type Types.VARCHAR mapped to String.
     */
    public static final int ID_NAME = 1;
    public static final int TYPE_NAME = Types.VARCHAR;
    public static final String NAME_NAME = "name";

    /**
     * Column type of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TYPE = 2;
    public static final int TYPE_TYPE = Types.VARCHAR;
    public static final String NAME_TYPE = "type";

    /**
     * Column length of type Types.SMALLINT mapped to Integer.
     */
    public static final int ID_LENGTH = 3;
    public static final int TYPE_LENGTH = Types.SMALLINT;
    public static final String NAME_LENGTH = "length";

    /**
     * Column format of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FORMAT = 4;
    public static final int TYPE_FORMAT = Types.VARCHAR;
    public static final String NAME_FORMAT = "format";

    /**
     * Column keyword of type Types.VARCHAR mapped to String.
     */
    public static final int ID_KEYWORD = 5;
    public static final int TYPE_KEYWORD = Types.VARCHAR;
    public static final String NAME_KEYWORD = "keyword";

    /**
     * Column mandatory of type Types.TINYINT mapped to Integer.
     */
    public static final int ID_MANDATORY = 6;
    public static final int TYPE_MANDATORY = Types.TINYINT;
    public static final String NAME_MANDATORY = "mandatory";


    private static final String TABLE_NAME = "calib_columns";

    /**
     * Create an array of type string containing all the fields of the calib_columns table.
     */
    private static final String[] FIELD_NAMES = 
    {
        "calib_columns.tname"
        ,"calib_columns.name"
        ,"calib_columns.type"
        ,"calib_columns.length"
        ,"calib_columns.format"
        ,"calib_columns.keyword"
        
    };

    /**
     * Field that contains the comma separated fields of the calib_columns table.
     */
    private static final String ALL_FIELDS = "calib_columns.tname"
                            + ",calib_columns.name"
                            + ",calib_columns.type"
                            + ",calib_columns.length"
                            + ",calib_columns.format"
                            + ",calib_columns.keyword";

    private static CalibColumnsDAO singleton = new CalibColumnsDAO();

    /**
     * Get the CalibColumnsManager singleton.
     *
     * @return CalibColumnsManager 
     */
    synchronized public static CalibColumnsDAO getInstance()
    {
        return singleton;
    }

    /**
     * Sets your own CalibColumnsManager instance.
     <br>
     * This is optional, by default we provide it for you.
     */
    synchronized public static void setInstance(CalibColumnsDAO instance)
    {
        singleton = instance;
    }


    /**
     * Creates a new CalibColumnsBean instance.
     *
     * @return the new CalibColumnsBean 
     */
    public MetaColumn createCalibColumnsBean()
    {
        return new MetaColumn();
    }



    //////////////////////////////////////
    // LOAD ALL
    //////////////////////////////////////

    /**
     * Loads all the rows from calib_columns.
     *
     * @return an array of CalibColumnsManager pObject
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //38
    public MetaColumn[] loadAll() throws SQLException, JdbcConnectionException 
    {
        Connection c = null;
        PreparedStatement ps = null;
        try 
        {
            c = getConnection();
            ps = c.prepareStatement("SELECT " + ALL_FIELDS + " FROM calib_columns",ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            return loadByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }

    //////////////////////////////////////
    // SQL 'WHERE' METHOD
    //////////////////////////////////////
    /**
     * Retrieves an array of CalibColumnsBean given a sql 'where' clause.
     *
     * @param where the sql 'where' clause
     * @return the resulting CalibColumnsBean table 
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //49
    public MetaColumn[] loadByWhere(String where) throws SQLException, JdbcConnectionException
    {
        return loadByWhere(where, null);
    }

    /**
     * Retrieves an array of CalibColumnsBean given a sql where clause, and a list of fields.
     * It is up to you to pass the 'WHERE' in your where clausis.
     *
     * @param where the sql 'where' clause
     * @param fieldList table of the field's associated constants
     * @return the resulting CalibColumnsBean table 
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //51
    public MetaColumn[] loadByWhere(String where, int[] fieldList) throws SQLException, JdbcConnectionException
    {
        String sql = null;
        if(fieldList == null)
            sql = "select " + ALL_FIELDS + " from calib_columns " + where;
        else
        {
            StringBuffer buff = new StringBuffer(128);
            buff.append("select ");
            for(int i = 0; i < fieldList.length; i++)
            {
                if(i != 0)
                    buff.append(",");
                buff.append(FIELD_NAMES[fieldList[i]]);
            }
            buff.append(" from calib_columns ");
            buff.append(where);
            sql = buff.toString();
            buff = null;
        }
        Connection c = null;
        Statement pStatement = null;
        ResultSet rs =  null;
        java.util.ArrayList v = null;
        try 
        {
            c = getConnection();
            pStatement = c.createStatement();
            rs =  pStatement.executeQuery(sql);
            v = new java.util.ArrayList();
            while(rs.next())
            {
                if(fieldList == null)
                    v.add(decodeRow(rs));
                else
                    v.add(decodeRow(rs, fieldList));
            }

            return (MetaColumn[])v.toArray(new MetaColumn[0]);
        }
        finally
        {
            if (v != null) { v.clear(); }
            getManager().close(pStatement, rs);
            freeConnection(c);
        }
    }


    /**
     * Deletes all rows from calib_columns table.
     * @return the number of deleted rows.
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public int deleteAll() throws SQLException, JdbcConnectionException
    {
        return deleteByWhere("");
    }


    /**
     * Deletes rows from the calib_columns table using a 'where' clause.
     * It is up to you to pass the 'WHERE' in your where clausis.
     * <br>Attention, if 'WHERE' is omitted it will delete all records. 
     *
     * @param where the sql 'where' clause
     * @return the number of deleted rows
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public int deleteByWhere(String where) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;

        try
        {
            c = getConnection();
            String delByWhereSQL = "DELETE FROM calib_columns " + where;
            ps = c.prepareStatement(delByWhereSQL);
            return ps.executeUpdate();
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // SAVE 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Saves the CalibColumnsBean pObject into the database.
     *
     * @param pObject the CalibColumnsBean pObject to be saved
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //100
    public MetaColumn save(MetaColumn pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer _sql = null;

        try
        {
            c = getConnection();
            if (pObject.isNew())
            { // SAVE 
                beforeInsert(pObject); // listener callback
                int _dirtyCount = 0;
                _sql = new StringBuffer("INSERT into calib_columns (");
    
                if (pObject.isTnameModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("tname");
                    _dirtyCount++;
                }

                if (pObject.isNameModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("name");
                    _dirtyCount++;
                }

                if (pObject.isTypeModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("type");
                    _dirtyCount++;
                }

                if (pObject.isLengthModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("length");
                    _dirtyCount++;
                }

                if (pObject.isFormatModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("format");
                    _dirtyCount++;
                }

                if (pObject.isKeywordModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("keyword");
                    _dirtyCount++;
                }



                _sql.append(") values (");
                if(_dirtyCount > 0) {
                    _sql.append("?");
                    for(int i = 1; i < _dirtyCount; i++) {
                        _sql.append(",?");
                    }
                }
                _sql.append(")");

                ps = c.prepareStatement(_sql.toString());
                _dirtyCount = 0;

                if (pObject.isTnameModified()) {
                    ps.setString(++_dirtyCount, pObject.getTname());
                }
    
                if (pObject.isNameModified()) {
                    ps.setString(++_dirtyCount, pObject.getName());
                }
    
                if (pObject.isTypeModified()) {
                    ps.setString(++_dirtyCount, pObject.getType());
                }
    
                if (pObject.isLengthModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getLength());
                }
    
                if (pObject.isFormatModified()) {
                    ps.setString(++_dirtyCount, pObject.getFormat());
                }
    
                if (pObject.isKeywordModified()) {
                    ps.setString(++_dirtyCount, pObject.getKeyword());
                }
    

    
                ps.executeUpdate();
    
                pObject.isNew(false);
                pObject.resetIsModified();
                afterInsert(pObject); // listener callback
            }
            else 
            { // UPDATE 
                beforeUpdate(pObject); // listener callback
                _sql = new StringBuffer("UPDATE calib_columns SET ");
                boolean useComma=false;

                if (pObject.isTnameModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("tname").append("=?");
                }

                if (pObject.isNameModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("name").append("=?");
                }

                if (pObject.isTypeModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("type").append("=?");
                }

                if (pObject.isLengthModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("length").append("=?");
                }

                if (pObject.isFormatModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("format").append("=?");
                }

                if (pObject.isKeywordModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("keyword").append("=?");
                }


                _sql.append("");
                ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                int _dirtyCount = 0;

                if (pObject.isTnameModified()) {
                      ps.setString(++_dirtyCount, pObject.getTname());
                }

                if (pObject.isNameModified()) {
                      ps.setString(++_dirtyCount, pObject.getName());
                }

                if (pObject.isTypeModified()) {
                      ps.setString(++_dirtyCount, pObject.getType());
                }

                if (pObject.isLengthModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getLength());
                }

                if (pObject.isFormatModified()) {
                      ps.setString(++_dirtyCount, pObject.getFormat());
                }

                if (pObject.isKeywordModified()) {
                      ps.setString(++_dirtyCount, pObject.getKeyword());
                }

 
    
                if (_dirtyCount == 0) {
                     return pObject;
                }
    
                ps.executeUpdate();
                pObject.resetIsModified();
                afterUpdate(pObject); // listener callback
            }
    
            return pObject;
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    /**
     * Saves an array of CalibColumnsBean pObjects into the database.
     *
     * @param pObjects the CalibColumnsBean pObject table to be saved
     * @return the saved CalibColumnsBean array.
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //65
    public MetaColumn[] save(MetaColumn[] pObjects) throws SQLException, JdbcConnectionException 
    {
        for (int iIndex = 0; iIndex < pObjects.length; iIndex ++){
            save(pObjects[iIndex]);
        }
        return pObjects;
    }



    ///////////////////////////////////////////////////////////////////////
    // USING TEMPLATE 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Loads a unique CalibColumnsBean pObject from a template one giving a c
     *
     * @param pObject the CalibColumnsBean pObject to look for
     * @return the pObject matching the template
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //85
    public MetaColumn loadUniqueUsingTemplate(MetaColumn pObject) throws SQLException, JdbcConnectionException
    {
         MetaColumn[] pReturn = loadUsingTemplate(pObject);
         if (pReturn.length == 0)
             return null;
         if (pReturn.length > 1)
             throw new SQLException("More than one element !!");
         return pReturn[0];
     }

    /**
     * Loads an array of CalibColumnsBean from a template one.
     *
     * @param pObject the CalibColumnsBean template to look for
     * @return all the CalibColumnsBean matching the template
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //88
    public MetaColumn[] loadUsingTemplate(MetaColumn pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer where = new StringBuffer("");
        StringBuffer _sql = new StringBuffer("SELECT " + ALL_FIELDS + " from calib_columns WHERE ");
        StringBuffer _sqlWhere = new StringBuffer("");
        try
        {
            int _dirtyCount = 0;
    
             if (pObject.isTnameModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("tname= ?");
             }
    
             if (pObject.isNameModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("name= ?");
             }
    
             if (pObject.isTypeModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("type= ?");
             }
    
             if (pObject.isLengthModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("length= ?");
             }
    
             if (pObject.isFormatModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("format= ?");
             }
    
             if (pObject.isKeywordModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("keyword= ?");
             }
    

    
             if (_dirtyCount == 0) {
                 throw new SQLException ("The pObject to look for is invalid : not initialized !");
             }
             _sql.append(_sqlWhere);
             c = getConnection();
             ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
             _dirtyCount = 0;
    
             if (pObject.isTnameModified()) {
                 ps.setString(++_dirtyCount, pObject.getTname());
             }
    
             if (pObject.isNameModified()) {
                 ps.setString(++_dirtyCount, pObject.getName());
             }
    
             if (pObject.isTypeModified()) {
                 ps.setString(++_dirtyCount, pObject.getType());
             }
    
             if (pObject.isLengthModified()) {
                 DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getLength());
             }
    
             if (pObject.isFormatModified()) {
                 ps.setString(++_dirtyCount, pObject.getFormat());
             }
    
             if (pObject.isKeywordModified()) {
                 ps.setString(++_dirtyCount, pObject.getKeyword());
             }
    

    
             ps.executeQuery();
             return loadByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }
    /**
     * Deletes rows using a CalibColumnsBean template.
     *
     * @param pObject the CalibColumnsBean object(s) to be deleted
     * @return the number of deleted objects
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //63
    public int deleteUsingTemplate(MetaColumn pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer sql = null;
    
        try 
        {
            sql = new StringBuffer("DELETE FROM calib_columns WHERE ");
            int _dirtyAnd = 0;
            if (pObject.isTnameInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("tname").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isNameInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("name").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isTypeInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("type").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isLengthInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("length").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isFormatInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("format").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isKeywordInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("keyword").append("=?");
                _dirtyAnd ++;
            }
    

    
            c = getConnection();
            ps = c.prepareStatement(sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            int _dirtyCount = 0;
    
            if (pObject.isTnameInitialized()) {
                ps.setString(++_dirtyCount, pObject.getTname());
            }
    
            if (pObject.isNameInitialized()) {
                ps.setString(++_dirtyCount, pObject.getName());
            }
    
            if (pObject.isTypeInitialized()) {
                ps.setString(++_dirtyCount, pObject.getType());
            }
    
            if (pObject.isLengthInitialized()) {
                DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getLength());
            }
    
            if (pObject.isFormatInitialized()) {
                ps.setString(++_dirtyCount, pObject.getFormat());
            }
    
            if (pObject.isKeywordInitialized()) {
                ps.setString(++_dirtyCount, pObject.getKeyword());
            }
    

    
            int _rows = ps.executeUpdate();
            return _rows;
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // COUNT 
    ///////////////////////////////////////////////////////////////////////

    /**
     * Retrieves the number of rows of the table calib_columns.
     *
     * @return the number of rows returned
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //78
    public int countAll() throws SQLException, JdbcConnectionException
    {
        return countWhere("");
    }



    /**
     * Retrieves the number of rows of the table calib_columns with a 'where' clause.
     * It is up to you to pass the 'WHERE' in your where clausis.
     *
     * @param where the restriction clause
     * @return the number of rows returned
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public int countWhere(String where) throws SQLException, JdbcConnectionException
    {
        String sql = "select count(*) as MCOUNT from calib_columns " + where;
        Connection c = null;
        Statement pStatement = null;
        ResultSet rs =  null;
        try 
        {
            int iReturn = -1;    
            c = getConnection();
            pStatement = c.createStatement();
            rs =  pStatement.executeQuery(sql);
            if (rs.next())
            {
                iReturn = rs.getInt("MCOUNT");
            }
            if (iReturn != -1)
                return iReturn;
        }
        finally
        {
            getManager().close(pStatement, rs);
            freeConnection(c);
        }
       throw new SQLException("Error in countWhere");
    }

    /**
     * Retrieves the number of rows of the table calib_columns with a prepared statement.
     *
     * @param ps the PreparedStatement to be used
     * @return the number of rows returned
     */
    //82
    int countByPreparedStatement(PreparedStatement ps) throws SQLException
    {
        ResultSet rs =  null;
        try 
        {
            int iReturn = -1;
            rs = ps.executeQuery();
            if (rs.next())
                iReturn = rs.getInt("MCOUNT");
            if (iReturn != -1)
                return iReturn;
        }
        finally
        {
            getManager().close(rs);
        }
       throw new SQLException("Error in countByPreparedStatement");
    }

    /**
     * Looks for the number of elements of a specific CalibColumnsBean pObject given a c
     *
     * @param pObject the CalibColumnsBean pObject to look for
     * @return the number of rows returned
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //83
    public int countUsingTemplate(MetaColumn pObject) throws SQLException, JdbcConnectionException
    {
        StringBuffer where = new StringBuffer("");
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer _sql = null;
        StringBuffer _sqlWhere = null;
    
        try
        {
                _sql = new StringBuffer("SELECT count(*) as MCOUNT  from calib_columns WHERE ");
                _sqlWhere = new StringBuffer("");
                int _dirtyCount = 0;
    
                if (pObject.isTnameModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("tname= ?");
                }
    
                if (pObject.isNameModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("name= ?");
                }
    
                if (pObject.isTypeModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("type= ?");
                }
    
                if (pObject.isLengthModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("length= ?");
                }
    
                if (pObject.isFormatModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("format= ?");
                }
    
                if (pObject.isKeywordModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("keyword= ?");
                }

    
                if (_dirtyCount == 0)
                   throw new SQLException ("The pObject to look is unvalid : not initialized !");
    
                _sql.append(_sqlWhere);
                c = getConnection();
                ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
    
                _dirtyCount = 0;
    
                if (pObject.isTnameModified()) {
                    ps.setString(++_dirtyCount, pObject.getTname());
                }
    
                if (pObject.isNameModified()) {
                    ps.setString(++_dirtyCount, pObject.getName());
                }
    
                if (pObject.isTypeModified()) {
                    ps.setString(++_dirtyCount, pObject.getType());
                }
    
                if (pObject.isLengthModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getLength());
                }
    
                if (pObject.isFormatModified()) {
                    ps.setString(++_dirtyCount, pObject.getFormat());
                }
    
                if (pObject.isKeywordModified()) {
                    ps.setString(++_dirtyCount, pObject.getKeyword());
                }
    

    
                return countByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // DECODE RESULT SET 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Transforms a ResultSet iterating on the calib_columns on a CalibColumnsBean pObject.
     *
     * @param rs the ResultSet to be transformed
     * @return pObject resulting CalibColumnsBean pObject
     */
    //72
    public MetaColumn decodeRow(ResultSet rs) throws SQLException
    {
        MetaColumn pObject = createCalibColumnsBean();
        pObject.setTname(rs.getString(1));
        pObject.setName(rs.getString(2));
        pObject.setType(rs.getString(3));
        pObject.setLength(DbConnManager.getInteger(rs, 4));
        pObject.setFormat(rs.getString(5));
        pObject.setKeyword(rs.getString(6));
        

        pObject.isNew(false);
        pObject.resetIsModified();

        return pObject;
    }

    /**
     * Transforms a ResultSet iterating on the calib_columns table on a CalibColumnsBean pObject according to a list of fields.
     *
     * @param rs the ResultSet to be transformed
     * @param fieldList table of the field's associated constants
     * @return pObject resulting CalibColumnsBean pObject
     */
    //73
    public MetaColumn decodeRow(ResultSet rs, int[] fieldList) throws SQLException
    {
        MetaColumn pObject = createCalibColumnsBean();
        int pos = 0;
        for(int i = 0; i < fieldList.length; i++)
        {
            switch(fieldList[i]) {
                case ID_TNAME:
                    ++pos;
                    pObject.setTname(rs.getString(pos));
                    break;
                case ID_NAME:
                    ++pos;
                    pObject.setName(rs.getString(pos));
                    break;
                case ID_TYPE:
                    ++pos;
                    pObject.setType(rs.getString(pos));
                    break;
                case ID_LENGTH:
                    ++pos;
                    pObject.setLength(DbConnManager.getInteger(rs, pos));
                    break;
                case ID_FORMAT:
                    ++pos;
                    pObject.setFormat(rs.getString(pos));
                    break;
                case ID_KEYWORD:
                    ++pos;
                    pObject.setKeyword(rs.getString(pos));
                    break;

            }
        }
        pObject.isNew(false);
        pObject.resetIsModified();

        return pObject;
    }

    //////////////////////////////////////
    // PREPARED STATEMENT LOADER
    //////////////////////////////////////

    /**
     * Loads all the elements using a prepared statement.
     *
     * @param ps the PreparedStatement to be used
     * @return an array of CalibColumnsBean 
     */
    //41
    public MetaColumn[] loadByPreparedStatement(PreparedStatement ps) throws SQLException
    {
        return loadByPreparedStatement(ps, null);
    }

    /**
     * Loads all the elements using a prepared statement specifying a list of fields to be retrieved.
     *
     * @param ps the PreparedStatement to be used
     * @param fieldList table of the field's associated constants
     * @return an array of CalibColumnsBean 
     */
    public MetaColumn[] loadByPreparedStatement(PreparedStatement ps, int[] fieldList) throws SQLException
    {
        ResultSet rs =  null;
        java.util.ArrayList v =  null;
        try
        {
            rs =  ps.executeQuery();
            v = new java.util.ArrayList();
            while(rs.next())
            {
                if(fieldList == null)
                    v.add(decodeRow(rs));
                else 
                    v.add(decodeRow(rs, fieldList));
            }
            return (MetaColumn[])v.toArray(new MetaColumn[0]);
        }
        finally
        {
            if (v != null) { v.clear(); v = null;}
            getManager().close(rs);
        }
    }

    ///////////////////////////////////////////////////////////////////////
    // LISTENER 
    ///////////////////////////////////////////////////////////////////////
    private CalibColumnsListener listener = null;

    /**
     * Registers a unique CalibColumnsListener listener.
     */
    //66.5
    public void registerListener(CalibColumnsListener listener) {
        this.listener = listener;
    }

    /**
     * Before the save of the CalibColumnsBean pObject.
     *
     * @param pObject the CalibColumnsBean pObject to be saved
     */
    //67
    void beforeInsert(MetaColumn pObject) throws SQLException {
        if (listener != null)
            listener.beforeInsert(pObject);
    }

    /**
     * After the save of the CalibColumnsBean pObject.
     *
     * @param pObject the CalibColumnsBean pObject to be saved
     */
    //68
    void afterInsert(MetaColumn pObject) throws SQLException {
        if (listener != null)
            listener.afterInsert(pObject);
    }

    /**
     * Before the update of the CalibColumnsBean pObject.
     *
     * @param pObject the CalibColumnsBean pObject to be updated
     */
    //69
    void beforeUpdate(MetaColumn pObject) throws SQLException {
        if (listener != null)
            listener.beforeUpdate(pObject);
    }

    /**
     * After the update of the CalibColumnsBean pObject.
     *
     * @param pObject the CalibColumnsBean pObject to be updated
     */
    //70
    void afterUpdate(MetaColumn pObject) throws SQLException {
        if (listener != null)
            listener.afterUpdate(pObject);
    }

    ///////////////////////////////////////////////////////////////////////
    // UTILS  
    ///////////////////////////////////////////////////////////////////////

    /**
     * Retrieves the manager object used to get connections.
     *
     * @return the manager used
     */
    //2
    DbConnManager getManager() {
        return DbConnManager.getInstance();
    }

    /**
     * Frees the connection.
     *
     * @param c the connection to release
     */
    void freeConnection(Connection c) {
        //getManager().releaseCalibConnection(c); // back to pool
    }
    /**
     * Gets the connection.
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    Connection getConnection() throws SQLException, JdbcConnectionException {
        return getManager().getCalibConnection();
    }

// class+ 

// class- 
}
