package org.eso.oca.data;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Collection;
import java.util.Enumeration;
import java.util.Iterator;

import org.apache.log4j.Logger;

/**
 * Class OCATableDAO
 * 
 * This class handles the creation of a table in a database and its manipulation
 * It contains a definition of the table and allow creating it, inserting,
 * updating and selecting
 * 
 * It assumes that all the tables will have a dp_id field as primary key, as
 * well as the columns are created given a set of keywords, which are converted
 * to acceptable column names
 * 
 * @version $Revision: 1.9 $
 * @author pnunes, Apr 25, 2005
 */

// $Id: OCATableDAO.java,v 1.9 2005/10/25 12:05:31 ochuzel Exp $
public class OCATableDAO extends TableDAO
{
    /*
     * Hard-coded properties for the dp_id type
     */
    private static final String dpIdField = "dp_id";

    private static final String dpIdType = "varchar(50)";

    private String classColType = "varchar(50) null";

    /*
     * Logger
     */
    private static Logger logger = Logger.getLogger(OCATableDAO.class);

    /**
     * Constructor
     * 
     * @param tableName
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public OCATableDAO(String tableName) throws SQLException,
            JdbcConnectionException
    {        
        super(tableName);
    }

    /**
     * Creates the temporary table using the table definition
     * 
     * @param keywords -
     *            contains a collection of keywords that will be the table
     *            columns
     * 
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public void create(String instrument, Collection keywords)
            throws SQLException, JdbcConnectionException
    {
        // clear table column definition
        clearTableColumns();

        // Creates the table definition
        // build table definition using the keywords,
        // adding each column to the table definition

        //Collection corpus = SimpleNode.getMetaKeywordCorpus();
        int i = 0;
        for (Iterator it = keywords.iterator(); it.hasNext();)
        {
            String keyword = (String) it.next();
            addTableColumn(keyword, classColType);
            i++;
        }

        //Build the SQL
        PreparedStatement ps = null;
        StringBuffer _sql = null;

        _sql = new StringBuffer("create table ");
        _sql.append(getTableName());
        _sql.append(" ( ");
        _sql.append(dpIdField + "  " + dpIdType);
        _sql.append(",");

        // iterate through the columns in the
        // hashtable and construct the statement
        Enumeration enumer = getColumns().keys();
        for (enumer = getColumns().keys(); enumer.hasMoreElements();)
        {
            String name = (String) enumer.nextElement();
            String type = (String) getColumns().get(name);
            _sql.append(name + "  " + type);
            _sql.append(",");
        }

        _sql.append(" primary key(" + dpIdField + ") ");
        _sql.append(" ) ");

        ps = getConnection().prepareStatement(_sql.toString());
        ps.executeUpdate();
        logger.debug(_sql.toString());

        //update db fits mapper as well
        getDbFitsMapper().addTable(getTableName(), instrument);
    }

    /**
     * inserts records in the temporary table with the dpId, leaving the
     * remaining columns null It inserts only dp_id found in the "data_products"
     * table
     * 
     * @param dpIds
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public void insertEmptyRecords(String[] dpIds) throws SQLException,
            JdbcConnectionException
    {
        PreparedStatement ps = null;
        StringBuffer _sql;

        for (int i = 0; i < dpIds.length; i++)
        {
            String dpId = dpIds[i];
            _sql = new StringBuffer();
            _sql.append("insert into " + getTableName() + " (" + dpIdField
                    + ") select dp_id from data_products where dp_id like '"
                    + dpId + "' ");
            ps = getConnection().prepareStatement(_sql.toString());
            ps.executeUpdate();
            logger.debug(_sql.toString());
        }
    }
}
