/* Generated By:JJTree: Do not edit this line. ASTDivision.java */

package org.eso.oca.parser;

import org.eso.oca.fits.TypedHeaderCard;
import org.eso.oca.fits.UndefinedFITSValue;

import java.lang.String;

import java.math.BigDecimal; 
import java.math.BigInteger; 

import org.apache.log4j.Logger;

public class ASTDivision extends SimpleNode
{
    public ASTDivision(int id)
    {
        super(id);
    }

    public ASTDivision(OcaParser p, int id)
    {
        super(p, id);
    }

    /** Accept the visitor. * 
     * @throws OcaVisitorException*/
    public Object jjtAccept(OcaParserVisitor visitor, Object data) throws OcaVisitorException
    {
        return visitor.visit(this, data);
    }

    public int precedence()
    {
        return MULTIPLICATION_PRECEDENCE;
    }
    
    static Logger logger = Logger.getLogger(ASTDivision.class);

    /** Precision available within a FITS keyword value of FLOAT type. */
    private static int FITS_PRECISION = 20; 
    /* 
     * The maximum sensible values for FITS_PRECISION are 20 for FIXED FLOAT 
     * format and 70 for FREE FLOAT format. The minimum value is zero. */ 

    /** Returns the exponent of the given BigDecimal's value. */
    private int exponent(BigDecimal number)
    {
        /* Get the length of the number's unscaled value as a string - this 
         * is the number of digits, plus an optional minus sign. 
         *
         * To do that, the unscaled value of the number is obtained as a 
         * BigInteger and converted to a String, then the string length 
         * is taken. 
         */ 
        int exponent = number.unscaledValue().toString().length(); 

        if(number.signum() < 0)
        {
            /* The number is negative - subtract 1 digit for the minus sign, 
             * plus 1 more digit to get the exponent of the unscaled value. 
             */
            exponent -= 2;
        }
        else
        {
            /* The number is not negative - subtract 1 digit to get the 
             * exponent of the unscaled value. 
             */ 
            --exponent; 
        }

        /* Subtract the number's scaling factor to get its exponent, and 
         * return the exponent. 
         */ 
        return(exponent - number.scale()); 
    }

    /** Computes the BigDecimal scaling factor required to hold the result of 
     * dividing leftOperand by rightOperand with FITS_PRECISION significant 
     * digits. 
     *
     * @see #FITS_PRECISION
     * @see java.math.BigDecimal 
     */
    private int divisionScale(BigDecimal leftOperand, BigDecimal rightOperand)
    {
        /* Compute the number of digits needed after the decimal point. */ 
        int scale = 
            exponent(rightOperand) - exponent(leftOperand) + FITS_PRECISION; 

        if(scale < 0)
        {
            /* For Java 1.4.x or earlier, the scale cannot be negative. This 
             * restriction means that BigDecimal operations with Java 1.5 
             * and up will be slightly less efficient than they could be, 
             * but they should still be correct. 
             */ 
            scale = 0;
        }

        return(scale); 
    }

    /** Generic fallback method for the dispatcher which catches attempts to 
     * divide invalid operands. Logs an error and throws an exception. 
     *
     * @throws InterpretationException in all cases. 
     */
    Object divide(Object leftOperand, Object rightOperand) 
        throws InterpretationException 
    {
        String message = getNodeLocationString() 
            + "Operator '/' may not be used for \"" 
            + TypedHeaderCard.findFITSTypeString(leftOperand) + " / " 
            + TypedHeaderCard.findFITSTypeString(rightOperand) + "\"."; 

        logger.error(message);
        throw new InterpretationException(message);
    }

    /** "Divides" an undefined value by any operand type; always returns an 
     * undefined result. 
     */
    UndefinedFITSValue divide(UndefinedFITSValue leftOperand, 
                              Object rightOperand) 
    {
        String message = getNodeLocationString() 
            + "Operator '/' used with undefined operand: \"" 
            + TypedHeaderCard.findFITSTypeString(leftOperand) + " / " 
            + TypedHeaderCard.findFITSTypeString(rightOperand) + "\".";

        logger.warn(message);
        return(new UndefinedFITSValue()); 
    }

    /** "Divides" any operand type by an undefined value; always returns an 
     * undefined result. 
     */
    UndefinedFITSValue divide(Object leftOperand, 
                              UndefinedFITSValue rightOperand) 
    {
        String message = getNodeLocationString() 
            + "Operator '/' used with undefined operand: \"" 
            + TypedHeaderCard.findFITSTypeString(leftOperand) + " / " 
            + TypedHeaderCard.findFITSTypeString(rightOperand) + "\".";

        logger.warn(message);
        return(new UndefinedFITSValue()); 
    }

    /** Divides one BigDecimal by another and returns the result. */
    BigDecimal divide(BigDecimal leftOperand, BigDecimal rightOperand)
    {
        return(leftOperand.divide(rightOperand, 
                                  divisionScale(leftOperand, rightOperand), 
                                  BigDecimal.ROUND_HALF_UP)); 
    }

    /** Divides a BigDecimal by a BigInteger and returns the result. The 
     * BigInteger is promoted to a BigDecimal first, and a warning is 
     * logged about the type mismatch. 
     */
    BigDecimal divide(BigDecimal leftOperand, BigInteger rightOperand)
    {
        logger.warn(getNodeLocationString() + "Performing '" 
                    + TypedHeaderCard.findFITSTypeString(leftOperand) + " / " 
                    + TypedHeaderCard.findFITSTypeString(rightOperand) + "'.");

        BigDecimal promotedRightOperand = new BigDecimal(rightOperand);
        return(leftOperand.divide(promotedRightOperand, 
                                  divisionScale(leftOperand, 
                                                promotedRightOperand), 
                                  BigDecimal.ROUND_HALF_UP)); 
    }

    /** Divides a BigInteger by a BigDecimal and returns the result. The 
     * BigInteger is promoted to a BigDecimal first, and a warning is 
     * logged about the type mismatch. 
     */
    BigDecimal divide(BigInteger leftOperand, BigDecimal rightOperand)
    {
        logger.warn(getNodeLocationString() + "Performing '" 
                    + TypedHeaderCard.findFITSTypeString(leftOperand) + " / " 
                    + TypedHeaderCard.findFITSTypeString(rightOperand) + "'.");

        BigDecimal promotedLeftOperand = new BigDecimal(leftOperand);
        return(promotedLeftOperand.divide(rightOperand, 
                                          divisionScale(promotedLeftOperand, 
                                                        rightOperand), 
                                          BigDecimal.ROUND_HALF_UP)); 
    }

    /** Divides one BigInteger by another and returns the result. */
    BigInteger divide(BigInteger leftOperand, BigInteger rightOperand)
    {
        return(leftOperand.divide(rightOperand)); 
    }

    public void interpret(OCAState state) throws InterpretationException 
    {
        logger.debug("interpret() called."); 

        jjtGetChild(0).interpret(state);
        jjtGetChild(1).interpret(state);

        /* Perform the operation by dispatching to the divide(l,r) methods 
         * defined above. Dispatching is based on the operand types. 
         */ 
        state.stack[--state.top] = 
            dispatch(this.getClass(), 
                     "divide", 
                     new Object[] {state.stack[state.top], 
                                   state.stack[state.top + 1]}); 
    }
}