package org.eso.oca.parser;

import java.lang.String;
import java.lang.Error;

/** This error is thrown by the OCA system's built-in interpreter if an 
 * irretrievable error occurs during reflection or dispatch. It is used 
 * to cover the so-called "impossible" exceptions that reflection should 
 * not be able to throw within the dispatcher, as well as runtime errors. 
 *
 * @author Neil Ferguson ESO/DMD/DFS <nferguso@eso.org> 
 */
public class DispatchReflectionError extends Error
{
    /** Creates a DispatchReflectionError with no detail message or cause. */ 
    public DispatchReflectionError()
    {
        super();
    }

    /** Creates a DispatchReflectionError with the specified detail message. 
     * 
     * @param message a detail message for this error. 
     */
    public DispatchReflectionError(String message)
    {
        super(message);
    }

    /** Creates a DispatchReflectionError with the specified detail message 
     * and cause. 
     *
     * @param message a detail message for this error. 
     * @param cause a Throwable which represents the underlying cause of 
     *              this error. 
     */
    public DispatchReflectionError(String message, Throwable cause)
    {
        super(message, cause);
    }

    /** Creates a DispatchReflectionError with the specified cause. 
     * 
     * @param cause a Throwable which represents the underlying cause of 
     *              this error. 
     */
    public DispatchReflectionError(Throwable cause)
    {
        super(cause);
    }
}
