/* Generated By:JJTree: Do not edit this line. SimpleNode.java */

package org.eso.oca.parser;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Collections;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.oca.backend.AssociationBlock;
import org.eso.oca.fits.OCAFile;
import org.eso.oca.fits.TypedHeaderCard;
import org.eso.oca.fits.UndefinedFITSValue;

/** SimpleNode is the base class for all OCA parser nodes.
 * It contains various static members used to share parsing
 * information (e.g. keywordCorpus) or interpretation information
 * (e.g. symtab) among the various nodes of the parse tree.
 * The methods classify, organize and associate are implemented
 * in this class.
 * The dispatch method, which is one of the main components
 * of the OCA interpreter, is also implemented here. 
 * 
 * @author szampier & nferguso
 *
 */

public class SimpleNode implements Node
{
    protected Node parent;

    protected Node[] children;

    protected int id;

    protected OcaParser parser;

    protected Token first, last;

    static Logger logger = Logger.getLogger(SimpleNode.class);

    public final int LOGICAL_OR_PRECEDENCE = 10;

    public final int LOGICAL_AND_PRECEDENCE = 20;

    public final int COMPARISON_PRECEDENCE = 30;

    public final int ADDITION_PRECEDENCE = 40;

    public final int MULTIPLICATION_PRECEDENCE = 50;

    public final int DEFAULT_PRECEDENCE = 100;

    public SimpleNode(int i)
    {
        id = i;
    }

    public SimpleNode(OcaParser p, int i)
    {
        this(i);
        parser = p;
    }

    public static Node jjtCreate(OcaParser p, int id)
    {
        return new SimpleNode(p, id);
    }

    public void jjtOpen()
    {
        first = parser.getToken(1);
    }

    public void jjtClose()
    {
        last = parser.getToken(0);
    }

    public Token getFirstToken()
    {
        return first;
    }

    public Token getLastToken()
    {
        return last;
    }

    public void jjtSetParent(Node n)
    {
        parent = n;
    }

    public Node jjtGetParent()
    {
        return parent;
    }

    public void jjtAddChild(Node n, int i)
    {
        if (children == null)
        {
            children = new Node[i + 1];
        } else if (i >= children.length)
        {
            Node c[] = new Node[i + 1];
            System.arraycopy(children, 0, c, 0, children.length);
            children = c;
        }
        children[i] = n;
    }

    public Node jjtGetChild(int i)
    {
        return children[i];
    }

    public int jjtGetNumChildren()
    {
        return (children == null) ? 0 : children.length;
    }

    /** Accept the visitor. * 
     * @throws OcaVisitorException*/
    public Object jjtAccept(OcaParserVisitor visitor, Object data) throws OcaVisitorException
    {
        return visitor.visit(this, data);
    }

    /** Accept the visitor. * 
     * @throws OcaVisitorException*/
    public Object childrenAccept(OcaParserVisitor visitor, Object data) throws OcaVisitorException
    {
        if (children != null)
        {
            for (int i = 0; i < children.length; ++i)
            {
                children[i].jjtAccept(visitor, data);
            }
        }
        return data;
    }

    public int precedence()
    {
        return DEFAULT_PRECEDENCE;
    }

    public String toString()
    {
        return OcaParserTreeConstants.jjtNodeName[id];
    }

    public String toString(String prefix)
    {
        return prefix + toString();
    }

    public void dump(String prefix)
    {
        System.out.println(toString(prefix));
        if (children != null)
        {
            for (int i = 0; i < children.length; ++i)
            {
                SimpleNode n = (SimpleNode) children[i];
                if (n != null)
                {
                    n.dump(prefix + " ");
                }
            }
        }
    }
    
    /** Returns an array of Strings containing all the keywords from the 
     * keyword corpus. Values must be stored for these keywords before the 
     * OCA interpreter can be used. 
     *
     * @see OCAState#updateSymtab() 
     */ 
    public String[] getRequestedData()
    {
        String[] requestedData = new String[getKeywordCorpus().size()];
        int i = 0;
        for (Iterator it = getKeywordCorpus().iterator(); it.hasNext();)
        {
            requestedData[i++] = (String) it.next();
        }
        return requestedData;
    }

    /** Returns a String stating where the parser was in the OCA rule file 
     * when it created this node. 
     */
    protected String getNodeLocationString() 
    {
        return("Rule file (Line " + getFirstToken().beginLine 
               + ", Column " + getFirstToken().beginColumn + "): "); 
    }

    public void interpret(OCAState state) throws InterpretationException 
    {
        logger.debug("SimpleNode interpret() method should never be called."); 

        throw new RuntimeException ("SimpleNode interpret() method should never be called.");
    }

    /** REDO ALL COMMENTS HEREIN... 
     */ 
    protected Object dispatch(Class callingClass, String methodName, Object[] args) 
        throws InterpretationException 
    {
        /*  */
        Method methodToCall = 
            DispatchHelper.getNearestMethod(
            	callingClass, 
                methodName, 
                args); 

        if(methodToCall == null)
        {
            /* There is no method that can be called on 'this' with the 
             * basic name in 'method' and which takes an appropriate set 
             * of argument types. 
             */ 
            String message = "dispatch() unable to find method."; 

            logger.debug(message); 
            throw new DispatchNoSuchMethodException(message); 
        }
        else
        {
            /* A suitable method was found - call it and return its result 
             * as the result of dispatch(). 
             */
            logger.debug("dispatch() found method, calling now."); 

            try
            {
                return(methodToCall.invoke(this, args));
            }
            catch(InvocationTargetException ite)
            {
                /* A method was dispatched to, but it threw an exception. */ 

                /* Wrap the original exception thrown by the method dispatched to 
                 * in an InterpretationException and throw that. 
                 */
                String message = 
                    "The method called by dispatch() threw an exception."; 

                logger.error(message); 
                throw new InterpretationException(message, ite.getCause()); 
            }
            catch(Exception e)
            {
                /* An error occurred during the reflection process. All of the 
                 * possible causes indicate bugs or runtime errors - wrap the 
                 * exception in a DispatchReflectionError and throw that. 
                 */ 
                String message = "An irretrievable error occurred during the " 
                    + "reflection and dispatching process. This may indicate " 
                    + "a bug. Please report it to an OCA maintainer."; 

                logger.error(message); 
                throw new DispatchReflectionError(message, e); 
            }
        }
    }

    /* *** BEGIN GENERIC METHODS FOR BOOLEAN LOGIC *** */

    /** Generic fallback method for the dispatcher which catches attempts 
     * to make invalid conversions to Boolean. Logs an error and throws an 
     * exception. 
     *
     * @throws InterpretationException in all cases. 
     */
    private Object makeBoolean(Object value) throws InterpretationException 
    {
        String message = getNodeLocationString() 
            + "Type '" + TypedHeaderCard.findFITSTypeString(value) 
            + "' may not be converted to 'BOOLEAN'."; 

        logger.error(message);
        throw new InterpretationException(message);
    }

    /** "Converts" UNDEFINED to Boolean (always returns false). */ 
    private Boolean makeBoolean(UndefinedFITSValue value)
    {
        String message = getNodeLocationString() 
            + "Converting '" + TypedHeaderCard.findFITSTypeString(value) 
            + "' to 'BOOLEAN'."; 

        logger.warn(message);
        return(new Boolean(false));
    }

    /** "Converts" Boolean to Boolean (actually returns the input value). */ 
    private Boolean makeBoolean(Boolean value)
    {
        return(value);
    }

    /** Converts the given Object to Boolean by dispatching to the appropriate 
     * conversion method (see above). Returns its result as an Object. 
     */
    protected Object getBoolean(Object value) throws InterpretationException 
    {
        return(dispatch(SimpleNode.class, "makeBoolean", new Object[] {value})); 
    }

    /* *** END GENERIC METHODS FOR BOOLEAN LOGIC *** */

    /** Returns the root node of this parse tree. */
    private ASTStart getTreeRoot()
    {
        Node currentNode = this;
        Node parentNode = this; 

        while(parentNode != null)
        {
            currentNode = parentNode;
            parentNode = currentNode.jjtGetParent(); 
        }

        /* The root node should be an instance of ASTStart - this throws a 
         * ClassCastException otherwise. 
         */
        return (ASTStart) currentNode; 
    }

    /** 
     * @return Wraps the keywordCorpus to prohibit modification and returns 
     * a reference to the wrapper object. 
     */
    public Collection getKeywordCorpus()
    {
        return 
            Collections.unmodifiableCollection(
                getTreeRoot().getModifiableKeywordCorpus()); 
    }

    /** 
     * @return Returns a reference to the keywordCorpus. 
     */
    Collection getModifiableKeywordCorpus()
    {
        return getTreeRoot().getModifiableKeywordCorpus(); 
    }

    /**
     * @return Wraps the metaKeywordCorpus to prohibit modification and 
     * returns a reference to the wrapper object. 
     */
    public Collection getMetaKeywordCorpus()
    {
        return 
            Collections.unmodifiableCollection(
                getTreeRoot().getModifiableMetaKeywordCorpus()); 
    }
    
    /**
     * @return Returns a reference to the metaKeywordCorpus. 
     */
    Collection getModifiableMetaKeywordCorpus()
    {
        return getTreeRoot().getModifiableMetaKeywordCorpus(); 
    }
    
    /** 
     * @return Wraps the classification rule set to prohibit modification 
     * and returns a reference to the wrapper object. 
     */
    public List getClassificationRules()
    {
        return 
            Collections.unmodifiableList(
                getTreeRoot().getModifiableClassificationRules()); 
    }

    /** 
     * @return Returns a reference to the classification rule set. 
     */
    List getModifiableClassificationRules()
    {
        return getTreeRoot().getModifiableClassificationRules(); 
    }

    /** 
     * @return Wraps the organisation rule set to prohibit modification 
     * and returns a reference to the wrapper object. 
     */
    public List getOrganisationRules()
    {
        return 
            Collections.unmodifiableList(
                getTreeRoot().getModifiableOrganisationRules()); 
    }

    /** 
     * @return Returns a reference to the organisation rule set. 
     */
    List getModifiableOrganisationRules()
    {
        return getTreeRoot().getModifiableOrganisationRules(); 
    }

    /** 
     * @return Wraps the association rule set to prohibit modification 
     * and returns a reference to the wrapper object. 
     */
    public Map getAssociationRules()
    {
        return 
            Collections.unmodifiableMap(
                getTreeRoot().getModifiableAssociationRules()); 
    }

    /** 
     * @return Returns a reference to the association rule set. 
     */
    Map getModifiableAssociationRules()
    {
        return getTreeRoot().getModifiableAssociationRules(); 
    }

    /**
     * Classify a set of OCA Files by adding meta keywords to the input files.
     * The modified input files are returned by this method.
     * NOTE: this method changes the input data.
     * 
     * @param files the files to classify.
     * @return input files after classification 
     * @throws InterpretationException
     */
    public OCAFile[] classify(OCAFile[] files) throws InterpretationException
    {
        OCAState state = new OCAState(); 
        state.opCounter = 0;

    	/* loop through the input files */
        for (int i=0; i<files.length; ++i)
        {
            /* update the symtab with the keyword values of the current file
             * a copy is made to avoid corrupting the input file
             */
            state.symtab = new Hashtable(files[i].getCards());
            /* loop through the classification rules */
            List classificationRules = getClassificationRules(); 
            for (int j = 0; j < classificationRules.size(); ++j)
            {
                ((ASTIfStatement) classificationRules.get(j)).interpret(state);
            }
            /* update the current file with the meta keyword values
             * created during the classification process
             */
            for (Iterator it = getMetaKeywordCorpus().iterator(); it.hasNext();)
            {
                /* The metakeyword. */
                String metaKey = (String) it.next();
                Object metaVal = null;
                
                /* The metakeyword value. */
                if((metaVal = state.symtab.get(metaKey)) != null) 
                    files[i].setMetaCard(metaKey, metaVal); 
                else 
                    files[i].setMetaCard(metaKey, new UndefinedFITSValue()); 
            }
            String msg;
            try {
                msg = files[i].getFilename() + " : " + files[i].getMetaCards().toString();
                logger.info(msg);
            } catch (Exception e) {
            }
        }
        logger.debug("operation count : "+state.opCounter);
        return files;
    }
    
    /**
     * Organise a set of OCA files into Association Blocks.
     * 
     * @param files
     * @return array of Association Blocks 
     * @throws InterpretationException
     */
    public AssociationBlock[] organize(OCAFile[] files) throws InterpretationException
    {
        OCAState state = new OCAState(); 
        state.opCounter = 0;
    	
    	/* the association blocks */
        Vector assocBlocks = new Vector();

        /* loop through the organisation rules */
        List organisationRules = getOrganisationRules();
        for (int i = 0; i < organisationRules.size(); ++i)
        {
            Vector selectedFiles = new Vector();
            ASTSelectExecuteStatement organisationRule = 
                (ASTSelectExecuteStatement) organisationRules.get(i); 

            /* loop through the files */
            for (int j = 0; j < files.length; ++j)
            {
                /* update the symtab with the keyword values of the current file
                 * a copy is made to avoid corrupting the input file
                 */
                state.symtab = new Hashtable(files[j].getCards());
                /* also meta-keywords can be used in the organization rules */
                state.symtab.putAll(files[j].getMetaCards());
                /* execute the current organisation rule
                 * the static flag isSelected is updated
                 */ 
                organisationRule.interpret(state);
                if (state.isSelected == true)
                    selectedFiles.add(files[j]);
            }
            String msg = selectedFiles.size() + " files selected by organization rule #" + i;
            if (selectedFiles.size() == 0) {
                logger.debug(msg);
            } else {
                logger.info(msg);
            }
            
            if (selectedFiles.size()==0)
                continue;
            
            /* get grouping keywords - raw match keywords */
            String[] matchKeywords =organisationRule.matchKeywords;
            OCAFile[] setOfFiles = (OCAFile[]) selectedFiles.toArray(new OCAFile[0]);
            /* if a grouping rule (raw match keywords) is defined
             * the current set of files can be split into groups
             */
            Vector groups = makeGroups(setOfFiles,matchKeywords);
            msg = groups.size() + " groups selected by organization rule #" + i;
            logger.info(msg);
            
            /* get the action to be invoked for the current set of files.
             * an action describes a recipe, a set of associated files and
             * the resulting products
             */
            String action = organisationRule.action;
            String groupEvent = organisationRule.groupEvent;
            String groupEventExt = organisationRule.groupEventExt;
            /* for each group, a new association block is created,
             * to be filled during the association process
             */
            for (Iterator it = groups.iterator(); it.hasNext(); )
            {
                AssociationBlock ab = new AssociationBlock();
                ab.setInputFiles((OCAFile[]) it.next());
                ab.setAction(action);
                ab.setGroupEvent(groupEvent);
                ab.setGroupEventExt(groupEventExt);
                ab.setMatchKeywords(matchKeywords);
                assocBlocks.add(ab);
            }
        }
        logger.debug("operation count : "+state.opCounter);
        return (AssociationBlock[]) assocBlocks.toArray(new AssociationBlock[0]);
    }
    
    /** The input set of OCA files is organized into homogeneous groups using
     * the group by clause (match keywords). A group is defined by a unique combination
     * of match keyword values. For each input file, a simple hash value is generated
     * by concatenating the match keyword values. If a group with the same hash value
     * already exists, the file is added to that group, otherwise a new group is
     * created.
     * 
     * @param files : the input set of files to organize 
     * @param matchKeywords : a set of FITS keywords used as grouping criteria
     * @return
     */
    private Vector makeGroups(OCAFile[] files, String[] matchKeywords)
    {
        Vector ret = new Vector();
        /* if no match keywords are given, just return 1 group containing all of the
         * input files
         */
        if (matchKeywords.length == 0)
        {
            ret.add(files);
            return ret;
        }
        /* we use an hash table as a container for the groups
         * - the key is the group hash value, obtained by concatenating all of the
         *   distinct values of the match keywords.
         * - the value is the list of files having the same match values as defined in
         *   the key
         */
        Hashtable groups = new Hashtable();
        for (int i=0; i<files.length; i++)
        {
            /* string buffer used to concatenate the match values */
            StringBuffer buf = new StringBuffer();
            for (int j=0; j<matchKeywords.length; j++)
            {
            	/* the match keyword */
                String matchKey = matchKeywords[j];
            	/* the match keyword value */
                String matchVal;
                try {
                    matchVal = files[i].getKeywordValue(matchKey);
                } catch (Exception e) {
                    /* TODO: handle KeywordNotFoundException
                     * separately from UndefinedValueException
                     */
                    matchVal = "NULL";
                }
                buf.append(matchKey+"="+matchVal+";");
            }
            /* this is the group "unique" identifier */
            String groupKey = buf.toString();
            if (groups.containsKey(groupKey))
            {
            	/* a group with this identifier already exists.
            	 * The current file is added to the group.
            	 */
            	((Vector) groups.get(groupKey)).add(files[i]);
                try {
                    logger.info("added "+files[i].getFilename()+" to current group");
                } catch (Exception e) {
                }
            }
            else
            {
            	/* a group with this identifier does not exist.
            	 * A new group is created and the current file is added
            	 * to the group.
            	 */
                Vector tmp = new Vector();
                tmp.add(files[i]);
                groups.put(groupKey,tmp);
                logger.info("created new group with match values : "+groupKey);
                try {
                    logger.info("added "+files[i].getFilename()+" to current group");
                } catch (Exception e) {
                }
            }
        }
        /* convert to a vector of OCAFile arrays */
        for (Iterator it = groups.keySet().iterator(); it.hasNext(); )
        {
            String key = (String) it.next();
            logger.debug("group key \""+key);
            Vector v = (Vector) groups.get(key);
            OCAFile[] of = (OCAFile[]) v.toArray(new OCAFile[0]);
            ret.add(of);
        }
        return ret;
    }
    
    /**
     * Update the input Association Blocks with association information.
     * NOTE: this method changes the input association blocks.    
     * 
     * @param assocBlocks : association blocks containing only input files
     * @param rawFiles : raw files that can be selected during the association process
     * @param calibFiles : calibration files that can be selected during the association process
     * @return association blocks containing input files + associated files
     * @throws InterpretationException
     */
    public AssociationBlock[] associate(AssociationBlock[] assocBlocks, OCAFile[] rawFiles, OCAFile[] calibFiles) throws InterpretationException
    {
        OCAState state = new OCAState(); 
        state.opCounter = 0;
        state.globCalibFiles = new Vector();
        state.globInputFiles = new Vector();

    	/* load the calibFiles "database" */
        for (int i=0; i<calibFiles.length; i++)
        {
            Hashtable ctab = new Hashtable(calibFiles[i].getCards());
            ctab.putAll(calibFiles[i].getMetaCards());
            
            /* add missing meta-keywords to calib file */
            String metaKeyword;
            for (Iterator it = getMetaKeywordCorpus().iterator(); it.hasNext();) {
                metaKeyword = (String) it.next();
                if (!ctab.containsKey(metaKeyword)) {
                    ctab.put(metaKeyword, new UndefinedFITSValue());
                }
            }
            
            /* add calib file to calibFiles */
            state.globCalibFiles.add(ctab);
        }
        
    	/* load the rawFiles "database" */
        for (int i=0; i<rawFiles.length; i++)
        {
            Hashtable rtab = new Hashtable(rawFiles[i].getCards());
            rtab.putAll(rawFiles[i].getMetaCards());
            
            /* add missing meta-keywords to raw file */
            String metaKeyword;
            for (Iterator it = getMetaKeywordCorpus().iterator(); it.hasNext();) {
                metaKeyword = (String) it.next();
                if (!rtab.containsKey(metaKeyword)) {
                    rtab.put(metaKeyword, new UndefinedFITSValue());
                }
            }
            
            /* add raw file to rawFiles */
            state.globInputFiles.add(rtab);
        }
        
        /* loop through the association blocks */
        Map associationRules = getAssociationRules();
        for (int i=0; i<assocBlocks.length; ++i)
        {
            /* get the action rule */
            ASTActionRule actionRule = (ASTActionRule) associationRules.get(assocBlocks[i].getAction());
            if (actionRule == null)
            {
                logger.error("undefined action "+assocBlocks[i].getAction());
                continue;
            }
            logger.info("invoking action "+actionRule.name+" for group "+i);
            /* update the input symtab with keyword and meta-keyword values
             * of the representative raw frame
             */
            state.inputFileSymtab = new Hashtable(assocBlocks[i].getExemplar().getCards());
            state.inputFileSymtab.putAll(assocBlocks[i].getExemplar().getMetaCards());
            try {
                state.inputFileSymtab.put(OCAFile.MJD_OBS_MOD, assocBlocks[i].getModifiedMjdObs());
            } catch (Exception e) {
                logger.debug("Can't add modified MJD-OBS to inputFileSymtab");
            }
            /* the associated files */
            state.associatedFiles = new Vector();
            state.globVirtualProducts = new Vector();
            state.isABValid = true;
            /* invoke the association rule interpreter */ 
            actionRule.interpret(state);
            OCAFile[] assoc = new OCAFile[state.associatedFiles.size()];
            for (int j=0; j<state.associatedFiles.size(); j++)
            {
                assoc[j] = new OCAFile((Hashtable) state.associatedFiles.elementAt(j));
            }
            OCAFile[] prods = new OCAFile[state.globVirtualProducts.size()];
            for (int j=0; j<state.globVirtualProducts.size(); j++)
            {
                prods[j] = new OCAFile((Hashtable) state.globVirtualProducts.elementAt(j));
            }
            /* update association block */
            assocBlocks[i].setAssociatedFiles(assoc);
            /* meta keywords are added to the products */
            assocBlocks[i].setProductFiles(prods);
            assocBlocks[i].setRecipe(state.globRecipe);
            assocBlocks[i].setRecipeParams(state.globRecipeParams);
            assocBlocks[i].setValid(state.isABValid);

            /* dirty fix : retrieve from the AB the keywords and meta-keywords 
             * (added by the AB) of the virtual products
             * 
             * The getProductFiles() call may seem redundant, given that the 
             * prods array is still available (see the setProductFiles() call 
             * above), but it is actually necessary, because the set and get 
             * trigger special manipulations on these values (see the method 
             * implementations within AssociationBlock). The manipulated 
             * versions are the ones that need to go into globCalibFiles. 
             */
            OCAFile[] modifiedProducts = assocBlocks[i].getProductFiles();
            for (int j = 0; j < modifiedProducts.length; ++j)
            {
            	Hashtable t = new Hashtable(modifiedProducts[j].getCards());
            	t.putAll(modifiedProducts[j].getMetaCards());
            	state.globCalibFiles.add(t);
            }
        }
        logger.debug("operation count : "+state.opCounter);
        return assocBlocks;
    }
}
