package org.eso.oca.parser;

import org.eso.oca.fits.TypedHeaderCard;

/**
 * This class contains some helper methods used by the SqlVisitor.
 * 
 * @author szampier
 */
public class SqlVisitorHelper {

	/**
	 * Converts the specified FITS keyword name to the
	 * corresponding database field name. Example:
	 * MJD-OBS -> mjd__obs
	 * DPR.CATG -> dpr_catg
	 * 
	 * @param key the FITS keyword name
	 * @return database field name
	 */
	public static String formatKeyForDB(String key) {
		String result = TypedHeaderCard.formatKey(key,
				TypedHeaderCard.SHORT_DOTTED);
		result = result.replace('.', '_');
		result = result.replace("-", "__");
		result = result.toLowerCase();
		if (result.equals("arcfile")) {
			result = "dp_id";
		}
		return result;
	}

	/**
	 * Converts the specified database field name to the
	 * corresponding FITS keyword name. Example:
	 * mjd__obs -> MJD-OBS
	 * dpr_catg -> DPR.CATG
	 * 
	 * @param key the database field name.
	 * @return FITS keyword name.
	 */
	public static String formatKeyFromDB(String key) {
		String result = key.replace("__", "-");
		result = result.replace('_', '.');
		result = result.toUpperCase();
		return result;
	}
}
