package org.eso.oca.properties;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Properties;

import org.apache.log4j.Logger;

/**
 * Class OCAProperties
 *
 * Singleton containing OCA Application properties
 * It reads them from a configuration file
 * 
 *
 * @version $Revision: 1.8 $
 * @author pnunes, Mar 30, 2005
 */

// $Id: OCAProperties.java,v 1.8 2005/06/06 10:32:04 szampier Exp $ 
public class OCAProperties {

    
    private static Logger logger = Logger.getLogger(OCAProperties.class);     
    
    private static Properties props; 
    
    /**
     * 
     */
    private static OCAProperties ocaProps;      

    /**
     * Constructs 
     */
    private OCAProperties() {}

    
    public void loadProperties(String cfFile) throws OCAPropertiesException {
        logger.debug("Loading properties from " + cfFile);
        load(cfFile);
    }
    
    // For lazy initialization 
    public static synchronized OCAProperties getInstance() { 
     if (ocaProps==null) { 
         ocaProps = new OCAProperties(); 
     } 
     return ocaProps; 
    } 
               
        
    /**
     * Load the properties from a file
     * @param filePath
     * @return boolean
     * @throws OCAPropertiesException
     */
    private boolean load(String filePath) throws OCAPropertiesException {
        
        props = new Properties();
        
        boolean debug = false;
        Enumeration e;
        FileInputStream fileStream = null;
        
        try {
            fileStream = new FileInputStream(filePath);
            // Note on Properties.load():
            // There is no exception thrown if the file is an invalid format
            // (such as a binary file) - simply no properties will be read in.
            props.load(fileStream);
        } catch (IOException ioe) {
            
            logger.debug("Failed to read properties from " + filePath);
            throw new OCAPropertiesException("Failed to read properties from " + filePath);            
        }
        
           
        return true;
    }

    
    /**
     * @param name
     * @return String containing the property value
     * @throws OCAPropertiesException
     * @throws OCAPropertiesException
     */
    public String getProperty(String name) throws OCAPropertiesException  {
        if ( props == null ) {            
            logger.debug("Properties not initialized. Trying to get property " + name );
            throw new OCAPropertiesException("Properties not initialized");
        }    
         String p = props.getProperty(name);
        
         if (p == null) {
             logger.debug("Failed to get property " + name );
             throw new OCAPropertiesException("Failed to get property " + name);
         }
         return(p);
    }

    
    
    

}





