package org.eso.util.dal;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import java.io.*;
import java.sql.Date;
import java.math.BigDecimal;
import java.util.List;
import java.util.ArrayList;

//import java.io.FileOutputStream;
//import java.io.File;
import javax.xml.parsers.*;
import org.w3c.dom.*;
//import org.w3c.dom.DOMConfiguration;
import org.w3c.dom.bootstrap.DOMImplementationRegistry;
import org.w3c.dom.ls.DOMImplementationLS;
import org.w3c.dom.ls.LSOutput;
import org.w3c.dom.ls.LSSerializer;
import java.io.FileWriter;

//import org.apache.xml.serialize.OutputFormat;
//import org.apache.xml.serialize.XMLSerializer;

public class NgasDiskDAOXML implements NgasDiskDAO {
	
	/** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(NgasDiskDAOXML.class);
    
    private Document d_document;
    
    private String	d_filename;
    
    private List<NgasDisk>	d_disks = new ArrayList<NgasDisk>(); 
	
	public NgasDiskDAOXML(String filename) throws DAOException {
		parseDocument(filename);
		d_filename = filename;
	}
	
	public void refreshView() throws DAOException {
		parseDocument(d_filename);
	}
	
	public List<NgasDisk> findAll() throws DAOException {
		return d_disks;
	}
	
	public List<NgasDisk> findByDiskId(String id) throws DAOException {
		List<NgasDisk> temp = new ArrayList<NgasDisk>();
		for (NgasDisk d: d_disks)
			if (d.getDiskId().equals(id))
				temp.add(d);
		return temp;
	}
	
	public List<NgasDisk> findByLogicalName(String name) throws DAOException {
		List<NgasDisk> temp = new ArrayList<NgasDisk>();
		for (NgasDisk d: d_disks)
			if (d.getLogicalName().equals(name))
				temp.add(d);
		return temp;
	}
	
	public List<NgasFile> findFilesByDiskId(String id) throws DAOException {
		throw new DAOException("Method not implemented in this DAO");
	}
	
	public NgasDisk findLastEntry() throws DAOException {
		throw new DAOException("Method not implemented in this DAO");
	}
	
	public void save(NgasDisk entry) throws DAOException {
		if (findByDiskId(entry.getDiskId()).size() != 0)
			throw new DAOException("disk_id " + entry.getDiskId() + " is already in the file");
		else if (d_disks.size() != 0)
			throw new DAOException("file already contains an entry, there can be at most one");
		else {
			d_disks.add(entry);
			saveToFile();
		}
	}
	
	public void saveIfNotFound(NgasDisk entry) throws DAOException {
		save(entry);
	}
	
	public void update(NgasDisk entry) throws DAOException {
		List<NgasDisk> temp = findByDiskId(entry.getDiskId()); 
		if (temp.size() == 1)
		{
			d_disks.set(d_disks.indexOf(temp.get(0)), entry);
			saveToFile();
		}
		else
			throw new DAOException("disk_id " + entry.getDiskId() + " is not in the file");
	}
	
	public void delete(NgasDisk entry) throws DAOException {
		if (findByDiskId(entry.getDiskId()).size() == 1)
		{
			d_disks.remove(entry);
			saveToFile();
		}
		else
			throw new DAOException("disk_id " + entry.getDiskId() + " is not in the file");
	}
	
	/**
	 * Parses an XML file and builds a Document used internally
	 * 
	 * @param filename the XML file
	 * @throws DAOException
	 */
	private void parseDocument(String filename) throws DAOException {
		//get the factory
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

		try {
			//Using factory get an instance of document builder
			DocumentBuilder db = dbf.newDocumentBuilder();

			//parse using builder to get DOM representation of the XML file
			d_document = db.parse(filename);
			
			//get the root element
			Element docEle = d_document.getDocumentElement();

			//get a nodelist of  elements
			NodeList nl = docEle.getElementsByTagName("Disk");
			//clear list
			d_disks.clear();
			if(nl != null && nl.getLength() > 0) {
				for(int i = 0 ; i < nl.getLength();i++) {

					//get the disk element
					Element el = (Element) nl.item(i);

					//get the disk object
					NgasDisk disk = getNgasDisk(el);

					//add it to list
					d_disks.add(disk);
				}
			}
		} catch(Exception e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	/**
	 * Converts an Element of the XML structure into an NgasDisk object
	 * 
	 * @param el the element to be converted
	 * @return the NgasDisk object
	 * @throws DAOException
	 */
	private NgasDisk getNgasDisk(Element el) throws DAOException {

		NgasDisk disk = new NgasDisk();
				
		disk.setDiskId(getTextValue(el, "disk_id"));
		if (disk.getDiskId() == null)
			throw new DAOException("missing disk_id");
		
		disk.setArchive(getTextValue(el, "archive"));
		try {
			disk.setInstallationDate(Date.valueOf(getTextValue(el, "installation_date")));
		} catch (Exception e) {}
		disk.setType(getTextValue(el, "type"));
		disk.setManufacturer(getTextValue(el, "manufacturer"));
		disk.setLogicalName(getTextValue(el, "logical_name"));
		disk.setHostId(getTextValue(el, "host_id"));
		disk.setSlotId(getTextValue(el, "slot_id"));
		try {
			disk.setMounted(Short.parseShort(getTextValue(el, "mounted")));
		} catch (Exception e) {}
		disk.setMountPoint(getTextValue(el, "mount_point"));
		try {
			disk.setNumberOfFiles(Integer.parseInt(getTextValue(el, "number_of_files")));
		} catch (Exception e) {}
		try {
			disk.setAvailableMb(Integer.parseInt(getTextValue(el, "available_mb")));
		} catch (Exception e) {}
		try {
			disk.setBytesStored(BigDecimal.valueOf(Float.valueOf(getTextValue(el, "bytes_stored"))));
		} catch (Exception e) {}
		try {
			disk.setCompleted(Short.parseShort(getTextValue(el, "completed")));
		} catch (Exception e) {}
		try {
			disk.setCompletionDate(Date.valueOf(getTextValue(el, "completion_date")));
		} catch (Exception e) {}
		disk.setChecksum(getTextValue(el, "checksum"));
		try {
			disk.setTotalDiskWriteTime(Float.parseFloat(getTextValue(el, "total_disk_write_time")));
		} catch (Exception e) {}
		try {
			disk.setLastCheck(Date.valueOf(getTextValue(el, "last_check")));
		} catch (Exception e) {}
		disk.setLastHostId(getTextValue(el, "last_host_id"));
		
		NgasDisk.validate(disk);
				
		return disk;
	}
	
	/**
	 * Extracts a String associated to a given tag from an Element
	 * 
	 * @param ele the XML element
	 * @param tagName the tag to be searched
	 * @return the value associated to the tag
	 */
	private String getTextValue(Element ele, String tagName) {
		String textVal = null;
		
		try {
			NodeList nl = ele.getElementsByTagName(tagName);
			if(nl != null && nl.getLength() > 0) {
				Element el = (Element)nl.item(0);
				textVal = el.getFirstChild().getNodeValue();
			}
		} catch (Exception e) {
		}

		return textVal;
	}
	
	/**
	 * Saves the document structure to disk
	 */
	private void saveToFile() {
		//get the factory
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

		try {
			//Using factory get an instance of document builder
			DocumentBuilder db = dbf.newDocumentBuilder();

			//parse using builder to get DOM representation of the XML file
			d_document = db.newDocument();
			
			//create the root element 
			Element rootEle = d_document.createElement("Disks");
			d_document.appendChild(rootEle);

			for (NgasDisk disk: d_disks) {
				Element diskEle = createDiskElement(disk);
				rootEle.appendChild(diskEle);
			}

			DOMImplementationRegistry registry = DOMImplementationRegistry.newInstance();
			DOMImplementationLS domImplLS = (DOMImplementationLS)registry.getDOMImplementation("LS");

			LSSerializer ser = domImplLS.createLSSerializer();  // Create a serializer for the DOM
			DOMConfiguration config = ser.getDomConfig();
			config.setParameter("format-pretty-print", true);
			
			LSOutput out = domImplLS.createLSOutput();
			FileWriter fileOut = new FileWriter(d_filename);
			out.setCharacterStream(fileOut);
			ser.write(d_document, out);                          // Serialize the DOM
			
			fileOut.close();
			
			/* TODO replace deprecated classes */
			/*OutputFormat format = new OutputFormat(d_document);
			format.setIndenting(true);

			XMLSerializer serializer = new XMLSerializer(new FileOutputStream(new File(d_filename)), format);
			serializer.serialize(d_document);*/
			
		} catch (Exception e) {
			logger.error("Error writing archive to disk: " + e.getMessage());
		}
	}
		
	/**
	 * Creates an XML Element from an NgasDisk instance
	 * 
	 * @param disk the disk to be converted 
	 * @return the XML Element
	 */
	private Element createDiskElement(NgasDisk disk) {
		Element diskEle = d_document.createElement("Disk");
		
		Element	el;
		Text	text;

		if (disk.getDiskId() != null) {
			el = d_document.createElement("disk_id");
			text = d_document.createTextNode(disk.getDiskId());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getArchive() != null) {
			el = d_document.createElement("archive");
			text = d_document.createTextNode(disk.getArchive());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getInstallationDate() != null) {
			el = d_document.createElement("installation_date");
			text = d_document.createTextNode(disk.getInstallationDate().toString());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getType() != null) {
			el = d_document.createElement("type");
			text = d_document.createTextNode(disk.getType());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getManufacturer() != null) {
			el = d_document.createElement("manufacturer");
			text = d_document.createTextNode(disk.getManufacturer());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getLogicalName() != null) {
			el = d_document.createElement("logical_name");
			text = d_document.createTextNode(disk.getLogicalName());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getHostId() != null) {
			el = d_document.createElement("host_id");
			text = d_document.createTextNode(disk.getHostId());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getSlotId() != null) {
			el = d_document.createElement("slot_id");
			text = d_document.createTextNode(disk.getSlotId());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getMounted() != null) {
			el = d_document.createElement("mounted");
			text = d_document.createTextNode(Short.toString(disk.getMounted()));
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getMountPoint() != null) {
			el = d_document.createElement("mount_point");
			text = d_document.createTextNode(disk.getMountPoint());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getNumberOfFiles() != null) {
			el = d_document.createElement("number_of_files");
			text = d_document.createTextNode(Integer.toString(disk.getNumberOfFiles()));
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getAvailableMb() != null) {
			el = d_document.createElement("available_mb");
			text = d_document.createTextNode(Integer.toString(disk.getAvailableMb()));
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getBytesStored() != null) {
			el = d_document.createElement("bytes_stored");
			text = d_document.createTextNode(disk.getBytesStored().toString());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getCompleted() != null) {
			el = d_document.createElement("completed");
			text = d_document.createTextNode(Short.toString(disk.getCompleted()));
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getCompletionDate() != null) {
			el = d_document.createElement("completion_date");
			text = d_document.createTextNode(disk.getCompletionDate().toString());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getChecksum() != null) {
			el = d_document.createElement("checksum");
			text = d_document.createTextNode(disk.getChecksum());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getTotalDiskWriteTime() != null) {
			el = d_document.createElement("total_disk_write_time");
			text = d_document.createTextNode(Float.toString(disk.getTotalDiskWriteTime()));
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getLastCheck() != null) {
			el = d_document.createElement("last_check");
			text = d_document.createTextNode(disk.getLastCheck().toString());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		if (disk.getLastHostId() != null) {
			el = d_document.createElement("last_host_id");
			text = d_document.createTextNode(disk.getLastHostId());
			el.appendChild(text);
			diskEle.appendChild(el);
		}
		return diskEle;
	}
	
	/**
	 * Initialize a file with an empty XML structure
	 * @param filename the filename to be created and initialized
	 * @throws IOException
	 */
	public static void initFile(String filename) throws IOException {
		BufferedWriter out = new BufferedWriter(new FileWriter(filename));
        out.write("<?xml version=\"1.0\"?>\n<Disks>\n</Disks>");
        out.close();
	}
	
	/**
	 * Test program
	 * @param args
	 */
	public static void main(String[] args) {
		BasicConfigurator.configure();
		try {
			//NgasDiskDAOXML.initFile("/media/sda1/disk.info");
			NgasDiskDAO dao = new NgasDiskDAOXML("/disk.info");
		
			for (NgasDisk d : dao.findAll())
			{
				try {
					NgasDisk.validate(d);
					System.out.println(d.toString());
					d.setNumberOfFiles(10);
					dao.update(d);
				} catch (DAOException e) {
					System.out.println(e.getMessage());
				}
			}
			
			NgasDisk d = new NgasDisk();
			d.setDiskId("2HA1WSKA");
			dao.update(d);
		} catch (Exception e) {
			System.out.println(e.getMessage());
		}
		
		return;		
	}
}
;