/**
 * 
 */
package org.eso.util.dal;

import java.util.List;

/**
 * NgasFileDAO interface.
 * 
 * @author mpadovan 
 */
public interface NgasFileDAO {

	/**
	 * Updates the ngas file entry in the repository with the given ngas file.
	 * @param ngasFile
	 * @return The number of updated entries.
	 * @throws DAOException
	 */
	public int update(NgasFile ngasFile) throws DAOException;

	/**
	 * Deletes the specified ngas file entry.
	 * @param ngasFile
	 * @return The number of deleted entries.
	 * @throws DAOException
	 */
	public int delete(NgasFile ngasFile) throws DAOException;
	
	/**
	 * Deletes all entries with the given disk_id.
	 * @param diskId
	 * @return The number of deleted entries.
	 * @throws DAOException
	 */
	public int deleteByDiskId(String diskId) throws DAOException;

	/**
	 * Deletes all entries with the given disk id and file id.
	 * @param diskId The disk id.
	 * @param fileId The file id.
	 * @return The number of deleted entries.
	 * @throws DAOException in case of error.
	 */
	public int deleteByDiskIdAndFileId(String diskId, String fileId) throws DAOException;

	/**
	 * Saves the ngas file in the repository.
	 * @param ngasFile
	 * @throws DAOException
	 */
	public void save(NgasFile ngasFile) throws DAOException;

	/**
	 * Saves the ngas file if not already in the repository.
	 * @param ngasFile
	 * @return <p>true if the ngas file was already in the repository</p>
	 *         <p>false if it was not in the repository</p>
	 * @throws DAOException
	 */
	public boolean saveIfNotFound(NgasFile ngasFile) throws DAOException;
	
	/**
	 * Returns all ngas files found in the repository.
	 * @return
	 * @throws DAOException
	 */
	public List<NgasFile> findAll() throws DAOException;

	/**
	 * Returns the ngas file identified by the specified parameters.  
	 * @param fileId
	 * @param diskId
	 * @param version
	 * @return
	 * @throws DAOException
	 */
	public List<NgasFile> find(String fileId, String diskId, Integer version) throws DAOException;

	/**
	 * Returns all the ngas files with the specified file ID.  
	 * @param fileId
	 * @return
	 * @throws DAOException
	 */
	public List<NgasFile> findByFileId(String fileId) throws DAOException;
	
    public Long getSize(List<String> fileIds) throws DAOException;
}
