/**
 * 
 */
package org.eso.util.dal;

import java.util.List;

/**
 * NgasFileSafeDAO interface.
 * 
 * @author Vincenzo Forchi` <vforchi@eso.org>
 */
public interface NgasFileSafeDAO {
	
	/**
	 * Updates the ngas file in the repository with the given ngas file.
	 * @param ngasFile
	 * @throws DAOException
	 */
	public void update(NgasFileSafe ngasFile) throws DAOException;

	/**
	 * Deletes the specified ngas file from the repository.
	 * @param ngasFile
	 * @throws DAOException
	 */
	public void delete(NgasFileSafe ngasFile) throws DAOException;

	/**
	 * Saves the ngas file safe in the repository.
	 * @param ngasFile
	 * @throws DAOException
	 */
	public void save(NgasFileSafe ngasFile) throws DAOException;

	/**
	 * Saves the ngas file safe if not already in the repository.
	 * @param ngasFile
	 * @return <p>true if the ngas file safe was already in the repository</p>
	 *         <p>false if it was not in the repository</p>
	 * @throws DAOException
	 */
	public boolean saveIfNotFound(NgasFileSafe ngasFile) throws DAOException;
	
	/**
	 * Returns all ngas file safes found in the repository.
	 * @return
	 * @throws DAOException
	 */
	public List<NgasFileSafe> findAll() throws DAOException;

	/**
	 * Returns the ngas file safe identified by the specified parameters.  
	 * @param fileId
	 * @param version
	 * @return
	 * @throws DAOException
	 */
	public List<NgasFileSafe> findByFileIdAndVersion(String fileId, Integer version) throws DAOException;

	/**
	 * Returns all the ngas file safes with the specified file ID.  
	 * @param fileId
	 * @return
	 * @throws DAOException
	 */
	public List<NgasFileSafe> findByFileId(String fileId) throws DAOException;
	
	/**
	 * Returns all the ngas file safes with the specified file ID and any version.
	 * @param fileId
	 * @return
	 * @throws DAOException
	 */
	public List<NgasFileSafe> findByFileIdIgnoreVersion(String fileId) throws DAOException;
}
