package org.eso.util.dal;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;

import org.apache.log4j.Logger;

public class PackageAuxFilesDAOSyb implements PackageAuxFilesDAO {

	/** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(PackageAuxFilesDAOSyb.class);
    
    /** The name of this class for logging purposes. */
	private static final String classLogName = "PackageAuxFilesDAOSyb";
	
	/* table columns */
	public static final String PACKAGEID_KW = "pkg_id";
	public static final String FILEID_KW = "file_id";
	public static final String AUXFILE_KW = "aux_file";
	
	private Connection	d_connection = null;
	private String		d_dbTable = null;
	
	public PackageAuxFilesDAOSyb(Connection conn, String dbTable) {
		d_connection 	= conn;
		d_dbTable		= dbTable;
	}
	
	public List<String> getAuxFiles(String fileId)
			throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE " + FILEID_KW + "='" + fileId + "'";
		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			List<String> result = new ArrayList<String>();
			while (rs.next()) {
				result.add(rs.getString(AUXFILE_KW));
			}
			return result;
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public Map<String ,List<String> > getAuxFiles(Integer pkgId)
			throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + "=" + pkgId;
		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			Map<String, List<String> > result = new HashMap<String, List<String>>();
			while (rs.next()) {
				String fileId = rs.getString(FILEID_KW); 
				if (!result.containsKey(fileId)) {
					result.put(fileId, new ArrayList<String>());
				}
				List<String> files = result.get(fileId);
				
				files.add(rs.getString(AUXFILE_KW));
			}
			return result;
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	public Map<String, List<String> > getAuxFiles(Integer pkgId, List<String> fileIds) throws DAOException {
    	
		Map<String, List<String> > res = new HashMap<String ,List<String> >();

    	/**
    	 * 1-100   => 1
    	 * 101-200 => 2
    	 * 201-300 => 3
    	 */
    	int numberOfIterations = (fileIds.size()+99)/100; 
    	
    	for (int i = 0; i < numberOfIterations; i++) {
    		int start = i * 100;
    		int stop = Math.min((i+1)*100, fileIds.size());
    		res.putAll(getAuxFiles(pkgId, fileIds, start, stop));
    	}

    	return res;			
	}
	
	private Map<String ,List<String> > getAuxFiles(Integer pkgId, List<String> fileIds, int start, int stop) throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + "=" + pkgId + " AND " + FILEID_KW + " IN (";
		for (int i = start; i < stop; i++) {
			query += "'" + fileIds.get(i) + "',";
		}
		query = query.substring(0, query.length()-1) + ")";
		
		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			Map<String, List<String> > result = new HashMap<String, List<String>>();
			while (rs.next()) {
				String fileId = rs.getString(FILEID_KW); 
				if (!result.containsKey(fileId)) {
					result.put(fileId, new ArrayList<String>());
				}
				List<String> files = result.get(fileId);

				files.add(rs.getString(AUXFILE_KW));
			}
			return result;
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public void insertPackageAuxFile(Integer pkgId, String fileId, String filename) throws DAOException {
		String query = "INSERT INTO " + d_dbTable + " (" + PACKAGEID_KW + "," + FILEID_KW + "," + AUXFILE_KW + ") ";
		query += " VALUES (" + pkgId + ",'" + fileId + "','" + filename + "')";
		try {
			SybaseConnectionService.executeStatement(d_connection, query);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	public void insertPackageAuxFiles(Integer pkgId, Map<String, List<String>> packageFiles) throws DAOException {
		List<String> statements = new ArrayList<String>();
		for (String fileId: packageFiles.keySet()) {
			for (String filename: packageFiles.get(fileId)) {
				String query = "INSERT INTO " + d_dbTable + " (" + PACKAGEID_KW + "," + FILEID_KW + "," + AUXFILE_KW + ") ";
				query += " VALUES (" + pkgId + ",'" + fileId + "','" + filename + "')";

				statements.add(query);
			}
		}
		
		try {
			SybaseConnectionService.executeBatchedStatement(d_connection, statements);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}	
	}
	
	public void insertPackageAuxFilesIfNotFound(Integer pkgId, Map<String, List<String>> packageFiles) throws DAOException {
		/* TODO: make it more efficient */
		Map<String, List<String>> toInsert = new HashMap<String, List<String>>();
		for (String fileId: packageFiles.keySet()) {
			List<String> temp = new ArrayList<String>();
			for (String filename: packageFiles.get(fileId)) {
				if (!findPackageAuxFile(pkgId, fileId, filename))
					temp.add(filename);
			}
			toInsert.put(fileId, temp);
		}
		insertPackageAuxFiles(pkgId, toInsert);
	}
	
	public boolean findPackageAuxFile(Integer pkgId, String fileId, String filename) throws DAOException {
		/* TODO: make it more efficient */
		String query = "SELECT * FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + " = " + pkgId;
		query += " AND "+ FILEID_KW + " = '" + fileId + "'";
		query += " AND "+ AUXFILE_KW + " = '" + filename + "'";
		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			boolean res = rs.next();
			return res;
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public void deletePackageAuxFile(Integer pkgId, String fileId,
			String filename) throws DAOException {
		String query = "DELETE FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + "=" + pkgId + " AND " 
			+ FILEID_KW + "='" + fileId + "' AND " + AUXFILE_KW + "='" + filename + "'";
		try {
			SybaseConnectionService.executeStatement(d_connection, query);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
		
	}

	public void deletePackageAuxFiles(Integer pkgId,
			Map<String, List<String>> packageFiles) throws DAOException {
		List<String> statements = new ArrayList<String>();
		for (String fileId: packageFiles.keySet()) {
			for (String filename: packageFiles.get(fileId)) {
				String query = "DELETE FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + "=" + pkgId + " AND " 
				+ FILEID_KW + "='" + fileId + "' AND " + AUXFILE_KW + "='" + filename + "'";

				statements.add(query);
			}
		}
		
		try {
			SybaseConnectionService.executeBatchedStatement(d_connection, statements);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
		
	}
}
