package org.eso.util.dal;

import java.sql.Timestamp;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Map;

import org.eso.util.dal.ReplicationRequest.Status;

/**
 * This class maps the table package_files, which describes the structure of PI packs 
 *
 * @author vforchi
 *
 */

public class PackageFile {
	public enum Type {
		RAW("R"),
		CALIBPROD("M"),
		SCIENCEPROD("S"),
		AUX("A");
		
		private static final Map<String, Type> lookup = new HashMap<String, Type>();

		static {
			for(Type s : EnumSet.allOf(Type.class))
				lookup.put(s.toString(), s);
		}
		
		private final String type;
		
		private Type(String type) {
			this.type = type;
		}
		
		@Override
		public String toString() {
			return type;
		}
		
		public static Type get(String type) {
			return lookup.get(type);
		}
		
		/* TODO: improve */
		public static Type fromName(String name) {
			if (name.substring(0,2).equals("A.")) 
				return AUX;
			else if (name.substring(0,2).equals("M.")) 
				return CALIBPROD;
			else if (name.substring(0,2).equals("S.")) 
				return SCIENCEPROD;
			else 
				return RAW;
		}
	}
	
	@Override
	public boolean equals(Object o2) {
		if (this == o2)
			return true;
		
		PackageFile p1 = this;
		PackageFile p2 = (PackageFile) o2;
		
		return (p1.getFilePath().equals(p2.getFilePath()) && p1.getFileName().equals(p2.getFileName()));
	}
	
	/**
	 * The package ID, defined as runid_instrument
	 */
	private Integer	d_packageId = null;
	
	/**
	 * The ingestion date the file
	 */
	private Timestamp	d_ingestionDate = null;

	/**
	 * The acquisition date the file
	 */
	private Timestamp	d_acquisitionDate = null;
	
	/**
	 * The file path relative to the root
	 * directory of the PI pack
	 */
	private String	d_filePath = null;
	
	/**
	 * The filename, without any directory
	 */
	private String	d_fileName = null;
	
	/**
	 * The file_id associated to the file in the archive
	 */
	private String	d_fileId = null;
	
	/**
	 * The file type 
	 */
	private Type d_fileType = null;

	public Integer getPackageId() {
		return d_packageId;
	}

	public void setPackageId(Integer id) {
		d_packageId = id;
	}

	public Timestamp getIngestionDate() {
		return d_ingestionDate;
	}

	public void setIngestionDate(Timestamp date) {
		d_ingestionDate = date;
	}

	public String getFileName() {
		return d_fileName;
	}

	public void setFileName(String name) {
		d_fileName = name;
	}

	public String getFileId() {
		return d_fileId;
	}

	public void setFileId(String id) {
		d_fileId = id;
	}

	public String getFilePath() {
		return d_filePath;
	}

	public void setFilePath(String path) {
		d_filePath = path;
	}

	public Type getFileType() {
		return d_fileType;
	}

	public void setFileType(Type type) {
		d_fileType = type;
	}

	public Timestamp getAcquisitionDate() {
		return d_acquisitionDate;
	}

	public void setAcquisitionDate(Timestamp date) {
		d_acquisitionDate = date;
	}

}
