package org.eso.util.dal;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.List;
import java.util.ArrayList;

import org.apache.log4j.Logger;

public class PackageFilesDAOSyb implements PackageFilesDAO {

	/** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(PackageFilesDAOSyb.class);
    
    /** The name of this class for logging purposes. */
	private static final String classLogName = "PackageFilesDAOSyb";
	
	/* table columns */
	public static final String PACKAGEID_KW = "pkg_id";
	public static final String INGESTIONDATE_KW = "ingestion_date";
	public static final String ACQUISITIONDATE_KW = "acquisition_date";
	public static final String FILENAME_KW = "file_name";
	public static final String FILEID_KW = "file_id";
	public static final String FILEPATH_KW = "file_path";
	public static final String FILETYPE_KW = "file_type";
	
	private Connection	d_connection = null;
	private String		d_dbTable = null;
	
	public PackageFilesDAOSyb(Connection conn, String dbTable) throws DAOException {
		d_connection 	= conn;
		d_dbTable		= dbTable;
	}
	
	public void delete(List<PackageFile> files) throws DAOException {
		List<String> statements = new ArrayList<String>();
		for (PackageFile file: files) {
			String query = "DELETE FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + "=" + file.getPackageId() + " AND " + FILEPATH_KW + "='" + file.getFilePath() + "' AND " + FILENAME_KW + "='" + file.getFileName() + "'";
			
			statements.add(query);
		}
		
		try {
			SybaseConnectionService.executeBatchedStatement(d_connection, statements);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}	
	}

	public List<PackageFile> find(Integer pkg_id, Timestamp start, Timestamp stop) throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + " = " + pkg_id;
		query += " AND " + ACQUISITIONDATE_KW + " BETWEEN dateadd(ss, " + (long)(start.getTime()*.001) + ", '01 Jan 1970') ";
		query += " AND dateadd(ss, " + (long)(stop.getTime()*.001) + ", '01 Jan 1970') ";
		
		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			List<PackageFile> files = new ArrayList<PackageFile>();
			
			while (rs.next()) {
				PackageFile file = new PackageFile();
				
				file.setPackageId(rs.getInt(PACKAGEID_KW));
				file.setIngestionDate(rs.getTimestamp(INGESTIONDATE_KW));
				file.setAcquisitionDate(rs.getTimestamp(ACQUISITIONDATE_KW));
				file.setFileName(rs.getString(FILENAME_KW));
				file.setFileId(rs.getString(FILEID_KW));
				file.setFilePath(rs.getString(FILEPATH_KW));
				file.setFileType(PackageFile.Type.get(rs.getString(FILETYPE_KW)));
				
				files.add(file);
			}
			
			return files;
			
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	public List<PackageFile> find(List<PackageFile> files) throws DAOException {
		/* return value */
		List<PackageFile> res = new ArrayList<PackageFile>();
	
		for (int i = 0; i <= files.size()/100; i++) {
			int start = i * 100;
			int stop = Math.min((i+1)*100, files.size());
			if (start != stop)
				res.addAll(find(files, start, stop));
		}
		
		return res;
	}
	
	private List<PackageFile> find(List<PackageFile> files, int startIdx, int stopIdx) throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE ";
		for (int i = startIdx; i < stopIdx; i++) {
			PackageFile file = files.get(i);
			query += " ( " + PACKAGEID_KW + " =  " + file.getPackageId();
			//query += " AND " + FILEID_KW + " = " + " '" + file.getFileId() + "') OR ";
			query += " AND " + FILEPATH_KW + " + " + FILENAME_KW + " = '" + file.getFilePath() + "' + '" + file.getFileName() + "') OR ";
		}
		query = query.substring(0, query.length()-4);

		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			List<PackageFile> res = new ArrayList<PackageFile>();

			while (rs.next()) {
				PackageFile file = new PackageFile();

				file.setPackageId(rs.getInt(PACKAGEID_KW));
				file.setIngestionDate(rs.getTimestamp(INGESTIONDATE_KW));
				file.setAcquisitionDate(rs.getTimestamp(ACQUISITIONDATE_KW));
				file.setFileName(rs.getString(FILENAME_KW));
				file.setFileId(rs.getString(FILEID_KW));
				file.setFilePath(rs.getString(FILEPATH_KW));
				file.setFileType(PackageFile.Type.get(rs.getString(FILETYPE_KW)));

				res.add(file);
			}

			return res;

		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public void insert(List<PackageFile> files) throws DAOException {
		
		List<String> statements = new ArrayList<String>();
		for (PackageFile file: files) {
		
			String columns = "(" + PACKAGEID_KW + "," + INGESTIONDATE_KW + "," + ACQUISITIONDATE_KW + "," + FILENAME_KW + "," + FILEID_KW + "," + FILETYPE_KW + "," + FILEPATH_KW + ")";
			
			String values = "(" + file.getPackageId() + ",'" + file.getIngestionDate() + "','" + file.getAcquisitionDate() + "','" + file.getFileName() + "','" + file.getFileId() + "','" + file.getFileType().toString() + "','" + file.getFilePath() + "')";
			
			String query = "INSERT INTO " + d_dbTable + " " + columns + " VALUES " + values;
			
			statements.add(query);
		}
		
		try {
			SybaseConnectionService.executeBatchedStatement(d_connection, statements);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public void insertIfNotFound(List<PackageFile> files) throws DAOException {
		List<PackageFile> notToInsert = find(files);
		List<PackageFile> toInsert = new ArrayList<PackageFile>();
		for (PackageFile f: files) {
			if (!notToInsert.contains(f)) {
				logger.info("Inserting [" + f.getFilePath() + System.getProperty("file.separator") + f.getFileName() + "]");
				toInsert.add(f);
			}
		}
		insert(toInsert);		
	}
	
	public boolean findByQCFilename(Integer pkgId, String filePath, String qcFilename) throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE ";
		//query += PACKAGEID_KW + " =  " + pkgId + " AND " + FILENAME_KW + " = " + " '" + qcFilename + "'";
		query += PACKAGEID_KW + " =  " + pkgId + " AND " + FILEPATH_KW + " = " + " '" + filePath + "'";
		query += " AND " + FILENAME_KW + " = " + " '" + qcFilename + "'";
		
		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
		
			if (rs.next())
				return true;
			else
				return false;
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}
}
