package org.eso.util.datatransfer;

import org.apache.log4j.Logger;
import org.eso.util.misc.SystemCommand;

/**
 * Implementation of a command that will compute a raw file checksum
 * 
 * @author cmoins
 * 
 */
public class ChecksumCommand extends SystemCommand {

    static final Logger logger = Logger.getLogger(UnpackCommand.class);
    /** The name of this class for logging purposes. */
    private static final String classLogName = "ChecksumCommand";

    private final String FILENAME_KEYWORD = "\\$FILENAME";

    /**
     * Constructor
     * 
     * @param checksumCmdFormat
     *            format of the command to compute a file checksum
     * @param fileName
     *            name of the file whose checksum is to be computed
     * @param timeout
     *            checksum command timeout
     */
    public ChecksumCommand(String checksumCmdFormat, String fileName,
            long timeout) {
        super();
        final String methodLogName = classLogName + "::ChecksumCommand()";
        if ((checksumCmdFormat == null) || (fileName == null)) {
            String message = methodLogName + " - Some parameters are missing";
            logger.error(message);
            throw new IllegalArgumentException(message);
        }
        String command = checksumCmdFormat;
        command = command.replaceAll(FILENAME_KEYWORD, fileName);
        setCommandLine(command);
        setTimeout(timeout);
    }

}
