package org.eso.util.requesthandler;

import java.util.HashMap;
import java.util.Map;

/**
 * Definition of the available option names that will be used by the user when
 * requesting options definitions : CDR, DVR, MAG, LAN, FTP, AUT, VOSTORE,
 * CALSEL, GZIP, GUNZIP, VIMREP, WFIPREVI, PREIMA, ESOFIXHD, BEOWFIPR,
 * ORIGNAME
 * 
 * @author cmoins
 * 
 */
public enum DatasetRequestOptionName {

    CDR("CDR"), DVR("DVR"), MAG("MAG"), LAN("LAN"), FTP("FTP"), AUT("AUT"), VOSTORE(
            "vostore"), CALSEL("calSel"), GZIP("gzip"), GUNZIP("gunzip"), VIMREP(
            "vimRep"), WFIPREVI("WFIPrevi "), PREIMA("PreIma"), ESOFIXHD(
            "esofixhd"), BEOWFIPR("beowfipr"), ORIGNAME("origname");

    private static final Map<String, DatasetRequestOptionName> INSTANCES =
            new HashMap<String, DatasetRequestOptionName>();

    static {
        DatasetRequestOptionName allOptionNames[] =
                DatasetRequestOptionName.values();
        for (int i = 0; i < allOptionNames.length; i++) {
            INSTANCES.put(allOptionNames[i].toString(), allOptionNames[i]);
        }
    }

    // String representation of the option
    private String name;

    DatasetRequestOptionName(String name) {
        this.name = name;
    }

    /** returns the string representation of an option */
    public String toString() {
        return name;
    }

    /** returns the option enum type element associated to a string */
    public static DatasetRequestOptionName getInstance(String name) {
        return INSTANCES.get(name.trim());
    }
}
