package org.eso.util.requesthandler;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.List;

import org.eso.util.requesthandler.ArchiveName;
import org.eso.util.requesthandler.DAOException;
import org.eso.util.requesthandler.DatasetRequestOption;
import org.eso.util.requesthandler.DatasetRequestOptionDAO;
import org.eso.util.requesthandler.DatasetRequestOptionName;
import org.springframework.test.AbstractDependencyInjectionSpringContextTests;

public class DatasetRequestOptionDAOTest extends
        AbstractDependencyInjectionSpringContextTests {

    private DatasetRequestOptionDAO requestOptionDAO;

    public DatasetRequestOptionDAOTest(String name) {
        super(name);
    }

    @Override
    protected String[] getConfigLocations() {
        // TODO Auto-generated method stub
        return new String[] { "config/spring/db.xml",
                "config/spring/hibernate.xml", "config/spring/dao.xml",
                "config/spring/transaction.xml" };
    }

    /** Nominal case for the find by medium and archive method */
    public void testFindByMediumAndArchive_01() {
        final DatasetRequestOptionName REQ_MEDIUM_TEST =
                DatasetRequestOptionName.CDR;
        final ArchiveName ARCHIVE_TEST = ArchiveName.SAF;
        try {
            DatasetRequestOption dsReqOpt =
                    requestOptionDAO.findByMediumAndArchive(REQ_MEDIUM_TEST,
                            ARCHIVE_TEST);
            assertNotNull(dsReqOpt);
        } catch (DAOException e) {
            fail("An exception was thrown" + e.getMessage());
        }
    }

    /** A non media option is given as a parameter */
    public void testFindByMediumAndArchive_02() {
        final DatasetRequestOptionName REQ_MEDIUM_TEST =
                DatasetRequestOptionName.CALSEL;
        final ArchiveName ARCHIVE_TEST = ArchiveName.SAF;
        try {
            requestOptionDAO.findByMediumAndArchive(REQ_MEDIUM_TEST,
                    ARCHIVE_TEST);
            fail("No exception raised");
        } catch (DAOException e) {
            // expected behaviour
        }
    }

    /** Nominal case for the find by option and archive method */
    public void testFindByOptionAndArchive_01() {

        final DatasetRequestOptionName REQ_OPTION_TEST =
                DatasetRequestOptionName.CALSEL;
        final ArchiveName ARCHIVE_TEST = ArchiveName.SAF;

        try {
            DatasetRequestOption dsReqOpt =
                    requestOptionDAO.findByOptionNameAndArchive(REQ_OPTION_TEST,
                            ARCHIVE_TEST);
            assertNotNull(dsReqOpt);
        } catch (DAOException e) {
            StringWriter sw = new StringWriter();
            e.printStackTrace(new PrintWriter(sw));
            fail("An exception was thrown" + sw + e.getMessage());
        }
    }

    public DatasetRequestOptionDAO getRequestOptionDAO() {
        return requestOptionDAO;
    }

    public void setRequestOptionDAO(DatasetRequestOptionDAO requestOptionDAO) {
        this.requestOptionDAO = requestOptionDAO;
    }

}
