package org.eso.dfs.io;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;

/**
 * @author nkornwei
 * Create/Launch an external process to extract and cat the file specified. The stdout
 * is read and returned through the Reader interface.
 * 
 * The objects are created through the createZCatReader() static method.
 */
public class ZCatReader extends BufferedReader {

    /**
     * The external command used to extract the file.
     */
    private static String zcatCommand = "zcat";

    private Process proc;

    public static ZCatReader createZCatReader(String dataAddress)
        throws IOException {
        String command = new String(zcatCommand + " " + dataAddress);
        Runtime r = Runtime.getRuntime();
        Process p = r.exec(command);
        // System.out.println("ZCatting " + dataAddress);
        return new ZCatReader(new InputStreamReader(p.getInputStream()), p);
    }

    private ZCatReader(Reader in, Process p) {
        super(in);
        proc = p;
    }

    /**
     * @see java.io.Reader#close()
     */
    public void close() throws IOException {
        // System.out.println("CLOSING ZCat Reader");
        // killing the process causes intermittent hangs - but (on SOLARIS) closing
        // the stdout pipe will send a SIGINT, which acheives the same
        // thing....so proc.destroy() is currently commented out.
        super.close();
        proc.destroy();

    }

    /**
     * Returns the zcatCommand.
     * @return String
     */
    public static String getZcatCommand() {
        return zcatCommand;
    }

    /**
     * Sets the zcatCommand.
     * @param zcatCommand The zcatCommand to set
     */
    public static void setZcatCommand(String zcatCommand) {
        ZCatReader.zcatCommand = zcatCommand;
    }

    public static void main(String[] args) {
        if (args.length == 0) {
            System.out.println("Please include .Z files to cat");
        } else {
            for (int i = 0; i < args.length; i++) {
                try {
                    java.util.Date start = new java.util.Date();
                    ZCatReader r = ZCatReader.createZCatReader(args[i]);
                    BufferedReader rdr = new BufferedReader(r);

                    char[] rec = new char[80];
                    while (rdr.read(rec) >= 0) {
                        //System.out.println("Read: "+new String(rec));
                    }
                    System.out.println(
                        "Read "
                            + args[i]
                            + " in: "
                            + ((new java.util.Date()).getTime()
                                - start.getTime())
                            + "ms.");
                    rdr.close();
                } catch (IOException e) {
                    System.out.println(
                        "Error catting file: " + args[i] + ": " + e.toString());
                }
            }
        }

    }
}
