package org.eso.gasgano.datamodel;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Enumeration;

import org.eso.gasgano.datamodel.gui.ResultsComponent;
import org.eso.gasgano.datamodel.gui.ResultsContainer;
import org.eso.gasgano.properties.PropertyDB;
import org.eso.gasgano.tools.SortedHashtable;

/**
 * this class is the main interface to the file provided
 * by tiraPack. This is the only class which should be
 * changed in case of format changes in the file<p>
 * @author: Michele Zamparelli
 */

public class ResultsModel {
    private SortedHashtable references = null;
    private int refCounter = 0;
    private String referenceLabel = "[REFERENCES]";

    public ResultsModel(BufferedReader br) {
        references = new SortedHashtable();
        init(br);
    }

    public ResultsModel() {
        references = new SortedHashtable();
        // temporary
    }

    public void init(BufferedReader buffRead) {
        int linesCount = 0;
        boolean debug = false;
        String tmpString = "";
        ResultsContainer tmpLabel, tmpRef, tmp2Label = null;

        if (PropertyDB.getInstance().isDebug(PropertyDB.debugDataModelChange)) {
            debug = true;
        }

        try {
            tmpString = buffRead.readLine();
            while (buffRead.ready()) {
                if (tmpString.startsWith(referenceLabel)) {
                    String refName = "Ref " + String.valueOf(refCounter);
                    tmpString = "";
                    tmpRef = addReference(refName);
                    if (debug)
                        System.out.println(
                            "Added Reference: << " + tmpRef + ">>");
                    do {
                        tmpString = buffRead.readLine();
                        tmpString.trim();
                        linesCount++;
                        if (tmpString.equals(""))
                            continue;

                            while (buffRead.ready()
                                && tmpString.startsWith("\t")) { // it's indented
                                if (tmpString.startsWith("\t[")) {
                                    // it's a level one Label
                                    tmpLabel = tmpRef.addLabel(tmpString);
                                    if (debug)
                                        System.out.println(
                                            "Added Label "
                                                + tmpString
                                                + " to "
                                                + tmpRef);
                                    tmpString = buffRead.readLine();
                                    tmpString.trim();
                                    linesCount++;
                                    // while it's not a level two label
                                    while (tmpString.startsWith("\t")
                                        && !tmpString.startsWith("\t[")
                                        && !tmpString.startsWith("\t\t[")) {
                                        if (!tmpString.equals("")) {
                                            tmpLabel.addFrame(tmpString);
                                            if (debug)
                                                System.out.println(
                                                    "Adding "
                                                        + tmpString
                                                        + " to "
                                                        + tmpLabel);
                                        }
                                        tmpString = buffRead.readLine();
                                        tmpString.trim();
                                        linesCount++;
                                    }

                                    while (buffRead.ready()
                                        && tmpString.startsWith("\t\t")) {
                                        // handling of level two here
                                        if (tmpString.startsWith("\t\t[")) {
                                            tmp2Label =
                                                tmpLabel.addLabel(tmpString);
                                            if (debug)
                                                System.out.println(
                                                    "Adding "
                                                        + tmp2Label
                                                        + " to "
                                                        + tmpLabel);
                                            //
                                            do {
                                                tmpString = buffRead.readLine();
                                                tmpString.trim();
                                                linesCount++;
                                                if (!tmpString.equals("")
                                                    && tmpString.startsWith(
                                                        "\t\t")
                                                    && !tmpString.startsWith(
                                                        "\t\t[")) {
                                                    tmp2Label.addFrame(
                                                        tmpString);
                                                    if (debug)
                                                        System.out.println(
                                                            "Adding "
                                                                + tmpString
                                                                + " to "
                                                                + tmp2Label);
                                                }
                                            } while (
                                                buffRead.ready()
                                                    && tmpString.startsWith(
                                                        "\t\t")
                                                    && !tmpString.startsWith(
                                                        "\t\t["));

                                        }
                                    }
                                }
                            } // it's level one loop end
                        if (!tmpString.startsWith(referenceLabel)) {
                            tmpRef.addFrame(tmpString);
                            if (debug)
                                System.out.println(
                                    "Adding " + tmpString + " to " + tmpRef);
                        }
                    }
                    while (!tmpString.startsWith(referenceLabel)
                        && buffRead.ready());
                } // it was a reference label
            } // buffer.ready()  while loop end
        } catch (IOException ioe) {
            System.err.println("gasgano: problem with reading results buffer");
        }
        //System.out.println("Read "+linesCount+" lines\n");
        //displayModel();
    }

    public void init2(BufferedReader buffRead) {
        int linesCount = 0;
        String tmpString = "";
        try {
            tmpString = buffRead.readLine();
            while (buffRead.ready()) {

                // System.out.println(tmpString);

                if (tmpString.startsWith(referenceLabel)) {
                    String refName = "Ref " + String.valueOf(refCounter);
                    ResultsContainer tmpRef = addReference(refName);
                    ResultsContainer tmpLabel = null;
                    tmpString = "";

                    do {
                        tmpString = buffRead.readLine();
                        linesCount++;
                        if (tmpString.equals("")) {
                            continue;
                        }
                        if (tmpString.startsWith("[")) {
                            if (!tmpString.startsWith(referenceLabel)) {
                                tmpLabel = tmpRef.addLabel(tmpString);
                                //System.out.println("Added Label "+tmpString);
                                tmpString = buffRead.readLine();
                                while (!tmpString.startsWith("[")) {
                                    if (!tmpString.equals("")) {
                                        tmpLabel.addFrame(tmpString);
                                        //System.out.println("Adding "+tmpString+" to "+tmpLabel);
                                    }
                                    tmpString = buffRead.readLine();
                                }
                            }
                        } else {
                            tmpRef.addFrame(tmpString);
                            //System.out.println("Added Frame "+tmpString);
                        }
                    }
                    while (!tmpString.startsWith(referenceLabel)
                        && buffRead.ready());
                }
            }
        } catch (IOException ioe) {
            System.err.println("gasgano: problem with reading results buffer");
        }
        //System.out.println("Read "+linesCount+" lines\n");
    }

    /**
     * temporary method, as long as cdpacker is not interfaced
     */

    public BufferedReader tmpAssessTiraPackInfo(String fileName) {
        BufferedReader buffRead = null;
        FileReader readerFile = null;
        File infoFile = null;
        String tmpString = "PACK.66.A-0597D.info";
        tmpString = "PACK.66.A-0307A.info";
        if (fileName != null) {
            tmpString = fileName;
        }
        infoFile = new File(tmpString);
        if (!(infoFile.exists() && infoFile.canRead())) {
            System.out.println("File non existent or not readable");
            System.exit(-1);
        }
        try {
            readerFile = new FileReader(tmpString);
            buffRead = new BufferedReader(readerFile);
        } catch (Exception e) {
            System.err.println(e);
        }
        return buffRead;
    }

    public void displayModel() {
        System.out.println("References: " + getReferenceCount());
        for (Enumeration e = references.elements(); e.hasMoreElements();) {
            ResultsContainer tmpCont = (ResultsContainer) e.nextElement();
            System.out.println(tmpCont);
            for (Enumeration ee = tmpCont.getFramesSet().elements();
                ee.hasMoreElements();
                ) {
                ResultsComponent tmpFrame = (ResultsComponent) ee.nextElement();
                System.out.println("   " + tmpFrame);
            }
            for (Enumeration ee = tmpCont.getLabelsSet().elements();
                ee.hasMoreElements();
                ) {
                ResultsContainer tmpLabel = (ResultsContainer) ee.nextElement();
                System.out.println("       " + tmpLabel);
                for (Enumeration eee = tmpLabel.getFramesSet().elements();
                    eee.hasMoreElements();
                    ) {
                    ResultsComponent tmpFrame =
                        (ResultsComponent) eee.nextElement();
                    System.out.println("              " + tmpFrame);

                }
            }
        }
    }

    public void cleanup() {
        refCounter = 0;
        references.clear();
    }

    public ResultsContainer findReference(String id) {
        return null;
    }

    public ResultsContainer findReference(SortedHashtable progSet, String id) {
        return (ResultsContainer) progSet.get(id);
    }

    public int getReferenceCount() {
        return references.size();
    }

    public ResultsContainer getReferenceAt(int i) {
        return (ResultsContainer) (references.elementAt(i));
    }

    public int getIndexOfReference(ResultsContainer ref) {
        return references.indexOf(ref);
    }

    public SortedHashtable getReferenceSet() {
        return references;
    }

    public ResultsContainer addReference(String id) {
        return addReference(references, id);
    }

    private ResultsContainer addReference(SortedHashtable progSet, String id) {
        ResultsContainer rc = findReference(progSet, id);
        if (rc == null) {
            rc = new ResultsContainer('r', id);
            progSet.put(id, rc);
            refCounter++;
            //rc.setFather(this);
        }
        return rc;
    }

    public static void main(String args[]) {
        File infoFile = null;
        FileReader readerFile = null;
        BufferedReader buffRead = null;
        ResultsModel resMod = null;

        if (args.length == 1) {
            System.out.println(args[0]);
            infoFile = new File(args[0]);
            if (!(infoFile.exists() && infoFile.canRead())) {
                System.out.println("File non existent or not readable");
                System.exit(-1);
            }
            try {
                readerFile = new FileReader(args[0]);
                System.out.println("File opened");
                buffRead = new BufferedReader(readerFile);
            } catch (Exception e) {
                System.err.println(e);
            }

            resMod = new ResultsModel(buffRead);
            resMod.displayModel();
        }
    }

}
