/**
 * @author M. Zamparelli
 * @version $Id: VirtualReductionBlockOBComponent.java,v 1.2 2004/06/09 11:53:38 nkornwei Exp $
 * A special kind of OBComponent which is actually not inserted in the DataModel,
 * but only instantiated by the DataModel class. Its binding resides in an index Hashtable.
 * The reason for having it as  a subclass of OBComponent is for displaying purposes.
 *<p>
 * Notice that the class is instantiated and added to indexes by the DataModel,
 * but takes care itself of its removal, and of notifying components that the binding should
 * be cut<p>
 *
 */
package org.eso.gasgano.datamodel.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.EmptyBorder;

import org.eso.dfs.gui.SwingFileChooser;
import org.eso.gasgano.datamodel.DataModelChangeListener;
import org.eso.gasgano.datamodel.DataModelEvent;
import org.eso.gasgano.datamodel.ReductionBlockCommon;
import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.datamodel.filesystem.DFSFile;
import org.eso.gasgano.properties.PropertyDB;

public class VirtualReductionBlockOBComponent
    extends OBComponent
    implements ReductionBlockCommon, DataModelChangeListener {
    private Vector rawFiles = null;
    private Vector masterCalibFiles = null;
    private Vector masterCalibSignatures = null;
    private Vector masterCalibNames, masterCalibResolvedNames = null;
    private Vector productFiles = null;
    private String productBaseName = null;
    private DFSDataModel dm = null;
    private StringBuffer vrbData = null;
    private String myInstrument, myRecipe = null;
    private VRBPanel panelGlob = null;
    static private JFileChooser fileChooser = new SwingFileChooser();
    static private JScrollPane scrollPane = new JScrollPane();
    /**
     * the constructor essentially relies upon the 
     * triggering component
     */

    public VirtualReductionBlockOBComponent(KeywordContainerOBComponent triggerComp) {
        Vector tmpVect = null;
        dm = DFSDataModel.getDataModel();
        rawFiles = triggerComp.getAssociatedRawFrames();
        tmpVect = triggerComp.getAssociatedMasterInfo();
        masterCalibNames = (Vector) tmpVect.elementAt(0);
        // triggerComp.getAssociatedMasterFrames();
        masterCalibSignatures = (Vector) tmpVect.elementAt(1);
        // triggerComp.getAssociatedMasterSignatures();
        productBaseName = triggerComp.getId();
        myInstrument = triggerComp.getInstrument();
        myRecipe = triggerComp.getAssociatedRecipeName();
        triggerComp.freeHeaderMemory();
        if (PropertyDB
            .getInstance()
            .isDebug(PropertyDB.debugDataModelChange)) {
            System.out.println("Added: new VRB from " + productBaseName);
        }

        insertInDataModel();
    }

    /**
     * @param e The event to be processed.
     * Either the Component we used to get prog/ob info has
     * changed or we had no such Component and were in the Unknown group, 
     * and new files have been added to the DataModel,
     * either way, we need to remove
     * this from the data model and re-add this.
     */
    public void dataModelChanged(DataModelEvent e) {
        if (e.getEvent() == DataModelEvent.FILE_DELETED
            || e.getEvent() == DataModelEvent.FILE_ADDED
            || e.getEvent() == DataModelEvent.FILE_UPDATED) {
            if (PropertyDB.getInstance().isDebug(PropertyDB.debugEvent)) {
                System.out.println(
                    productBaseName + " received dataModelChangeEvent.");
            }
            insertInDataModel();
        }
    }

    private void insertInDataModel() {
        Vector dataSourceSet = null;
        OBComponent dataSource = null;

        dataSourceSet =
            dm.getObComponentByBaseName(
                DFSFile.extractBaseName(productBaseName));
        if (dataSourceSet != null && dataSourceSet.isEmpty() == false) {
            Enumeration e = dataSourceSet.elements();
            while (dataSource == null && e.hasMoreElements()) {
                dataSource = (OBComponent) e.nextElement();
                if (dataSource != null
                    && dataSource instanceof FitsFileOBComponent == false) {
                    // we only want to find a FitsFile.
                    dataSource = null;
                }
            }
        } else {
            // I think here we should remove all references to this
            // VRB, namely removing it from the hashtable
            dm.removeAssociatedVRB(DFSFile.extractBaseName(productBaseName));
            if (PropertyDB
                .getInstance()
                .isDebug(PropertyDB.debugDataModelChange)) {
                System.out.println("Deleting: VRB from " + productBaseName);
            }

            Vector tmpSet = getInputFileNames();
            if (tmpSet != null && tmpSet.isEmpty() == false) {
                Enumeration e = tmpSet.elements();
                while (e.hasMoreElements()) {
                    String compName = (String) e.nextElement();
                    OBComponent obc = dm.getObComponentByName(compName);
                    if (obc != null) {
                        obc.removeAssociatedReductionBlock(this);
                    }
                }
            }
            // 3. Master Calib associations.
            dataSourceSet = masterCalibSignatures;
            if (dataSourceSet != null && dataSourceSet.isEmpty() == false) {
                Enumeration e = dataSourceSet.elements();
                while (e.hasMoreElements()) {
                    OBComponent obc =
                        dm.getObComponentBySignature((String) e.nextElement());
                    if (obc != null) {
                        obc.removeAssociatedReductionBlock(this);
                    }
                }
            }
            dm.removeDataModelChangeListener(this);
            return;
        }

        // Add this Component as an associated ReductionBlock for 
        // all components read from the ReductionBlock.

        // 1. Product associations:
        productFiles = new Vector();
        if (dataSourceSet != null && dataSourceSet.isEmpty() == false) {
            Enumeration e = dataSourceSet.elements();
            while (e.hasMoreElements()) {
                OBComponent obc = (OBComponent) e.nextElement();
                if (obc != null && obc.isPipeProduct()) {
                    //System.out.println("Associated to "+obc);
                    obc.addAssociatedReductionBlock(this);
                    productFiles.addElement(obc.getId());
                }
            }
        }

        // 2. Raw (input file) associations:
        dataSourceSet = getInputFileNames();
        if (dataSourceSet != null && dataSourceSet.isEmpty() == false) {
            Enumeration e = dataSourceSet.elements();
            while (e.hasMoreElements()) {
                String compName = (String) e.nextElement();
                OBComponent obc =
                    dm.getObComponentByFilesystemBaseName(compName);
                if (obc != null) {
                    //System.out.println("Associated to "+obc);
                    obc.addAssociatedReductionBlock(this);
                    dataSourceSet.setElementAt(
                        obc.getId(),
                        dataSourceSet.indexOf(compName));
                }
            }
        }
        // 3. Master Calib associations.
        masterCalibFiles = new Vector();
        masterCalibResolvedNames = new Vector(masterCalibNames.size());
        dataSourceSet = masterCalibSignatures;

        for (int i = 0; i < masterCalibNames.size(); i++) {
            OBComponent obcs, obcn = null;
            String name, md5Sig = null;
            name = (String) masterCalibNames.elementAt(i);
            md5Sig = (String) masterCalibSignatures.elementAt(i);
            obcs = dm.getObComponentBySignature(md5Sig);
            if (obcs != null) {
                obcs.addAssociatedReductionBlock(this);
                masterCalibFiles.addElement(obcs.getId());
            }

            obcn = dm.getObComponentByFilesystemBaseName(name);
            if (obcn != null && obcn == obcs) {
                masterCalibResolvedNames.addElement(obcn.getId());
            } else {
                if (obcs != null
                    && obcs.getSourceFile().getFileName().equals(name)) {
                    masterCalibResolvedNames.addElement(obcs.getId());
                } else {
                    masterCalibResolvedNames.addElement(name);
                }
            }
        }
    }

    public Vector getProductFileNames() {
        return productFiles;
    }

    public Vector getInputFileNames() {
        return rawFiles;
    }

    public Vector getMasterCalibFileNames() {
        return masterCalibFiles;
    }

    public String getId() {
        return "Virtual Reduction Block: ("
            + DFSFile.extractBaseName(productBaseName)
            + ")";
    }

    public int getStatus() {
        return DFSFile.UNKNOWN_STATE;
    }

    public DFSFile getSourceFile() {
        return null;
    }

    public int update() {
        return 1;
    }

    public JComponent getDisplayComponent() {
        JComponent rc = null;
        vrbData = new StringBuffer();
        Enumeration e = null;

        scrollPane.setHorizontalScrollBarPolicy(
            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scrollPane.setVerticalScrollBarPolicy(
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        vrbData.append("recipe: " + myRecipe + "\n\n");
        vrbData.append("instrument: " + myInstrument + "\n");
        vrbData.append("\n");
        vrbData.append(
            productBaseName.substring(
                0,
                productBaseName.lastIndexOf(File.separatorChar) + 1)
                + DFSFile.extractBaseName(productBaseName)
                + "\n\n");
        vrbData.append("{\n");
        e = getInputFileNames().elements();
        while (e.hasMoreElements()) {
            String compName = (String) e.nextElement();
            String classification = "";
            if (dm.getObComponentByName(compName)
                instanceof KeywordContainerOBComponent) {
                classification =
                    ((KeywordContainerOBComponent) dm.getObComponentByName(compName))
                        .getClassification();
            }
            vrbData.append(compName + " " + classification + "\n");
        }
        vrbData.append("}");
        vrbData.append("\n\n");

        vrbData.append("{\n");
        e = masterCalibResolvedNames.elements();
        while (e.hasMoreElements()) {
            String compName = (String) e.nextElement();

            String classification = "";
            if (dm.getObComponentByName(compName)
                instanceof KeywordContainerOBComponent) {
                classification =
                    ((KeywordContainerOBComponent) dm.getObComponentByName(compName))
                        .getClassification();
            }
            vrbData.append(compName + " " + classification + "\n");
        }
        vrbData.append("}");

        panelGlob = new VRBPanel(getId(), vrbData.toString());

        scrollPane.setViewportView(panelGlob);
        rc = scrollPane;
        return rc;
    }

    public JComponent getTreeCellComponent() {
        return new JLabel("PIPPO");
    }

    private boolean saveFile(File checkFile) {
        FileWriter fileOutputStream = null;
        String fileName = checkFile.getName();

        if (checkFile.exists() && checkFile.canWrite()) {
            if (JOptionPane
                .showConfirmDialog(
                    panelGlob,
                    "The target file " + fileName + " exists, overwrite ?",
                    "write VRB to file",
                    JOptionPane.OK_CANCEL_OPTION)
                != JOptionPane.OK_OPTION) {
                return false;
            }
        }

        if (checkFile.exists() && !checkFile.canWrite()) {
            JOptionPane.showMessageDialog(
                panelGlob,
                "The target file "
                    + fileName
                    + " exists and is write protected\n");
            return false;
        }

        try {
            fileOutputStream = new FileWriter(checkFile.getAbsolutePath());
            fileOutputStream.write(vrbData.toString(), 0, vrbData.length());
            fileOutputStream.close();
        } catch (IOException ioe) {

            return false;
        }
        return true;
    }

    private class VRBPanel extends JPanel implements ActionListener {
        JButton dumpToFile = null;
        JPanel tmpPanel, tmpPanel2 = null;
        String saveLabel = "save to file";
        public VRBPanel(String title, String data) {
            super(new BorderLayout());

            dumpToFile = new JButton(saveLabel);
            dumpToFile.setPreferredSize(new Dimension(130, 30));
            dumpToFile.addActionListener(VRBPanel.this);

            //dumpToFile.setSize(new Dimension(30,30));
            tmpPanel = new JPanel(new GridLayout(8, 1));
            tmpPanel.add(new JLabel(""));
            tmpPanel.add(dumpToFile);
            tmpPanel.setBorder(new EmptyBorder(4, 4, 4, 4));

            tmpPanel2 = new JPanel(new BorderLayout());

            add(new DFSTextFileDisplay(title, data), "Center");
            add(tmpPanel, "East");
        }

        public void actionPerformed(ActionEvent a) {
            if (a.getActionCommand() == saveLabel) {
                String fileName =
                    "VRB-" + DFSFile.extractBaseName(productBaseName) + ".rb";

                fileChooser.setDialogTitle(
                    "save virtual reduction block to a file");
                fileChooser.setApproveButtonText("save");
                fileChooser.setSelectedFile(new File(fileName));
                fileChooser.setMultiSelectionEnabled(false);

                if (fileChooser.showOpenDialog(VRBPanel.this)
                    == JFileChooser.APPROVE_OPTION) {
                    if (fileChooser.getSelectedFile() != null) {
                        saveFile(fileChooser.getSelectedFile());
                    }
                }
            }
        }
    } // end of inner class VRBPanel
}
