package org.eso.cpl.gui;

import java.io.File;
import javax.swing.filechooser.FileFilter;

/**
 * Simple implementation of {@link javax.swing.filechooser.FileFilter} 
 * which just looks at file extensions.
 *
 * @author   Mark Taylor
 */
public class ExtensionFileFilter extends FileFilter {

    private final String extension_;
    private final String description_;

    /**
     * Constructs a new ExtensionFileFilter.
     *
     * @param  extension  the string with which filenames must terminate
     *         to qualify for acceptance (including "." if appropriate)
     * @param  description  text description of this class of files
     */
    public ExtensionFileFilter(String extension, String description) {
        extension_ = extension;
        description_ = description + " (*" + extension + ")";
    }

    /**
     * Returns true if either the file is a directory or its name 
     * ends with this filter's extension.
     */
    public boolean accept(File file) {
        return file != null && 
               (file.isDirectory() || file.getName().endsWith(extension_));
    }

    public String getDescription() {
        return description_;
    }

    public boolean equals(Object other) {
        if (other == null || ! other.getClass().equals(getClass())) {
            return false;
        }
        else {
            ExtensionFileFilter off = (ExtensionFileFilter) other;
            return extension_.equals(off.extension_)
                && description_.equals(off.description_); 
        }
    }

    public int hashCode() {
        int result = 23;
        result = 17 * result + extension_.hashCode();
        result = 17 * result + description_.hashCode();
        return result;
    }
}
