package org.eso.cpl.gui;

import java.awt.Component;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import org.eso.cpl.AssociationUtils;
import org.eso.cpl.FrameList;
import org.eso.dfs.datatransfer.FileListTransferHandler;


/**
 * 
 * This class presents a FileListTransferHandler interface to a FrameTableModel,
 * permitting FileList to be imported and exported through drag-drop operations.
 * 
 * The import and export gui components are assumed to be JTable components containing
 * FrameTableModel models - if this is not the case, no action is taken by
 * this class.
 * 
 * The import operation requires use of the data service for file association and
 * grouping, the process to convert a "file" into a "frame". Files that can not
 * be classified or are not of the supported type are silently ignored.
 * 
 * On succcessfully importing some or all of a file list, the approriate table
 * model events are fired. 
 * 
 * @author Nick Kornweibel
 */
public class FrameTableModelTransferHandler extends FileListTransferHandler {
    
    /**
     * Constructor.
     * @param m the table model for import export.
     */
    public FrameTableModelTransferHandler() {
    }
    
    /**
     * Export a file list (List of File objects). The component is assumed to be
     * either a JTable or a JScrollPane containing a JTable,
     * from which the selected row data drives selection of the frames
     * from the FrameTableModel.
     * @see org.eso.dfs.datatransfer.FileListTransferHandler#exportFileList(javax.swing.JComponent)
     */
    protected List exportFileList(JComponent source) {
        List rc = null;
        rc = new ArrayList();
        if (source instanceof JScrollPane) {
            JScrollPane scroller = (JScrollPane)source;
            Component c = scroller.getViewport().getView();
            if (c != null && c instanceof JTable) {
                source = (JComponent)c;
            }
        }
        if (source instanceof JTable) {
            JTable src = (JTable)source;
            if (src.getModel() instanceof FrameTableModel) {
                FrameTableModel mdl = (FrameTableModel)src.getModel();
                // the table contains a FrameTableModel.
                // find selected rows
                int[] selection = src.getSelectedRows();
                rc = new ArrayList();
                for (int i = 0; i < selection.length; i++) {
                    rc.add(mdl.getFrame(selection[i]).getFile());
                }
            }
        }
        return rc;
    }

    /**
     * Import a file list (List of File objects). If the the component is not
     * a JTable with a FrameTableModel model, then nothing happens.
     * 
     * @see org.eso.dfs.datatransfer.FileListTransferHandler#importFileList(javax.swing.JComponent, java.util.List)
     */
    protected void importFileList(JComponent target, List files) {
        if (target instanceof JScrollPane) {
            JScrollPane scroller = (JScrollPane)target;
            Component c = scroller.getViewport().getView();
            if (c != null && c instanceof JTable) {
                target = (JComponent)c;
            }
        }
        if (target instanceof JTable) {
            JTable tbl = (JTable)target;
            if (tbl.getModel() instanceof FrameTableModel) {
                FrameTableModel mdl = (FrameTableModel)tbl.getModel();
                FrameList frameList =  mdl.getFrames();
                FrameList newFrames = AssociationUtils.classifyFrames(files);
                frameList.addAll(newFrames);
                // fire events to notify all that the table has changed.
                mdl.fireTableDataChanged();
            }
        }        
    }
}
