package org.eso.cpl.gui;

import java.util.List;
import org.eso.cpl.MarkedFrameList;

/**
 * Subclass of FrameTableModel which provides two additional flags
 * (boolean columns) for each row, called 'retain' and 'active'.
 *
 * @author   Mark Taylor
 */
public class MarkedFrameTableModel extends FrameTableModel {

    /**
     * Construct a new editable model from a frame list.
     * Default values of the active and retained flags are assigned.
     *
     * @param  frames  the frames represented by this model.  Subsequent 
     *         changes to this list will not affect this model
     */
    public MarkedFrameTableModel(MarkedFrameList frames) {
        this(frames, true);
    }

    /**
     * Construct a new model from a frame list.
     * Default values of the active and retained flags are assigned.
     *
     * @param  frames  the frames represented by this model.  Subsequent 
     *         changes to this list will not affect this model
     * @param  isEditable  whether this table is to be editable or not
     */
    public MarkedFrameTableModel(final MarkedFrameList frames,
                                 boolean isEditable) {
        super(frames, isEditable);
        // name the Active column "Include".
        DataColumn activeColumn = 
            new DataColumn("Include", true, Boolean.class, 15) {
                public Object getValue(int irow) {
                    return frames.isActive(frames.getFrame(irow)) 
                               ? Boolean.TRUE
                               : Boolean.FALSE;
                }
                public void setValue(int irow, Object value) {
                    frames.setActive(frames.getFrame(irow),
                                     Boolean.TRUE.equals(value));
                }
            };
        DataColumn retainedColumn = 
            new DataColumn("Retained", true, Boolean.class, 15) {
                public Object getValue(int irow) {
                    return frames.isRetained(frames.getFrame(irow))
                               ? Boolean.TRUE
                               : Boolean.FALSE;
                }
                public void setValue(int irow, Object value) {
                    frames.setActive(frames.getFrame(irow),
                                     Boolean.TRUE.equals(value));
                }
            };
        List cols = getDataColumns();
        cols.add(0, activeColumn);
        // hide the retained column...not needed at this stage.
        //cols.add(1, retainedColumn);
    }
}
