package org.eso.cpl.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import org.eso.cpl.Recipe;

/**
 * Panel which displays all known information about a Recipe.
 *
 * @author   Mark Taylor
 */
public class RecipeDisplay extends JPanel {

    private Recipe recipe_;
    private JPanel stack_ = new JPanel(new GridBagLayout());
    private int nrow;
    private final JTextField authorLabel_ = makeLineField();
    private final JTextArea copyrightLabel_ = makeMultiLineField();
    private final JTextField emailLabel_ = makeLineField();
    private final JTextField libraryLabel_ = makeLineField();
    private final JTextField nameLabel_ = makeLineField();
    private final JTextField synopsisLabel_ = makeLineField();
    private final JTextField versionLabel_ = makeLineField();
    private final JTextArea descriptionArea_ = makeMultiLineField();

    /**
     * Constructs a blank RecipeDisplay.
     */
    public RecipeDisplay() {
        setLayout(new BoxLayout(this, BoxLayout.X_AXIS));

        addLineItem("Library", libraryLabel_);
        addLineItem("Name", nameLabel_);
        addLineItem("Synopsis", synopsisLabel_);
        addLineItem("Version", versionLabel_);
        addLineItem("Author", authorLabel_);
        addLineItem("Email", emailLabel_);
        addLargeItem("Copyright", wrapInScroller(copyrightLabel_));
        addLargeItem("Description", wrapInScroller(descriptionArea_));

        // Place the stack.
        add(stack_);
    }

    /**
     * Displays a given recipe in this panel.
     *
     * @param  recipe  the recipe (may be <tt>null</tt> for blank display)
     */
    public void setRecipe(Recipe recipe) {
        boolean isNull = recipe == null;
        String author = null;
        String copyright = null;
        String email = null;
        String library = null;
        String name = null;
        String synopsis = null;
        String version = null;
        String description = null;
        if (recipe != null) {
            author = recipe.getAuthor();
            copyright = recipe.getCopyright();
            description = recipe.getDescription();
            email = recipe.getEmail();
            library = recipe.getLibrary().getName();
            name = recipe.getName();
            synopsis = recipe.getSynopsis();
            version = Long.toString(recipe.getVersion());
        }
 
        String blank = " ";
        authorLabel_.setText(author == null ? blank : author);
        copyrightLabel_.setText(copyright == null ? blank : copyright);
        descriptionArea_.setText(description == null ? blank : description);
        emailLabel_.setText(email == null ? blank : email);
        libraryLabel_.setText(library == null ? blank : library);
        nameLabel_.setText(name == null ? blank : name);
        synopsisLabel_.setText(synopsis == null ? blank : synopsis);
        versionLabel_.setText(version == null ? blank : version);
        descriptionArea_.setCaretPosition(0);
        //----------------------------------------
        // DS FEB 2007 - DFS02559 (use fixed font in gasgano help window) 
        int fontDefaultSize = descriptionArea_.getFont().getSize();
        descriptionArea_.setFont(new Font("Monospaced", Font.PLAIN, fontDefaultSize));
        //----------------------------------------
    }

    private void addLineItem(String name, Component label) {
        GridBagConstraints c = new GridBagConstraints();
        c.gridy = nrow++;
        c.weighty = 0.0;
        c.gridx = 1;       
        c.ipadx = 4;
        c.ipady = 2;

        c.anchor = GridBagConstraints.WEST;
        c.weightx = 0.0;
        addWithConstraints(new JLabel(" " + name + ":"), c);

        c.gridx = 3;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1.0;
        c.fill = GridBagConstraints.HORIZONTAL;
        addWithConstraints(label, c);
    }

    private void addLargeItem(String name, Component item) {
        GridBagConstraints c = new GridBagConstraints();
        c.gridy = nrow++;
        c.gridx = 1;
        c.ipadx = 4;
        c.ipady = 2;

        c.anchor = GridBagConstraints.NORTHWEST;
        c.weightx = 0.0;
        addWithConstraints(new JLabel(" " + name + ":"), c);

        c.gridx = 3;
        c.anchor = GridBagConstraints.NORTHWEST;
        c.weightx = 1.0;
        c.weighty = 1.0;
        c.fill = GridBagConstraints.BOTH;
        addWithConstraints(item, c);
    }

    private void addWithConstraints(Component comp, GridBagConstraints cons) {
        ((GridBagLayout) stack_.getLayout()).setConstraints(comp, cons);
        stack_.add(comp);
    }

    public Dimension getMaximumSize() {
        return new Dimension(Short.MAX_VALUE, getPreferredSize().height);
    }

    private static JTextField makeLineField() {
        JTextField lab = new JTextField(35);
        
        lab.setText(" ");
        Color fg = lab.getForeground();
        Color bg = lab.getBackground();
        lab.setEditable(false);
        lab.setForeground(fg);
        lab.setBackground(bg);
        return lab;
    }
    
    private static JTextArea makeMultiLineField() {
        JTextArea lab = new JTextArea();
        lab.setLineWrap(true);
        lab.setWrapStyleWord(true);
        lab.setText(" ");
        lab.setOpaque(true);
        lab.setEditable(false);
        return lab;
    }
    
    private static JScrollPane wrapInScroller(JTextArea lab) {
        JScrollPane dscroller = new JScrollPane(lab);
        dscroller.setPreferredSize(new Dimension(500, 128));
        return dscroller;
    }
}
