package org.eso.cpl.jni;

/**
 * Interface for objects which wish to be informed of messages from the CPL
 * system.
 * <p>
 * The specifications for how to use this interface are somewhat sketchy
 * since they rely on a part of CPL which is known to require some
 * reworking.  Implementors and users of this interface should expect
 * to have to do some rewriting in the future.
 *
 * @author   Mark Taylor
 */
public interface MessageListener {

    /**
     * Deals with a log message.
     *
     * @param   domain  the domain of the message
     * @param   msg     the message to log
     * @param   flags   an integer flag pertaining to this message; 
     *          the format of this is currently undefined
     */
    void logMessage(String domain, String msg, int flags);

    /**
     * Deals with a standard output message.
     *
     * @param  msg  the message
     */
    void outMessage(String msg);

    /**
     * Deals with a standard error message.
     *
     * @param  msg  the message
     */
    void errMessage(String msg);
}
