package org.eso.cpl.test;

import java.io.PrintStream;
import java.util.Iterator;

import org.eso.cpl.CPLException;
import org.eso.cpl.Frame;
import org.eso.cpl.FrameList;
import org.eso.cpl.Parameter;
import org.eso.cpl.Recipe;

/**
 * Provides some logging methods for JavaCPL.
 *
 * @author   Mark Taylor
 */
public class CPLDebug {

    PrintStream out_;

    public CPLDebug() {
        out_ = System.out;
    }

    public void log(String str) {
        out_.println(str);
    }

    public FrameList executeWithLogging(Recipe recipe, Parameter[] params, 
                                        FrameList inFrames)
            throws CPLException {
        out_.println();
        out_.println("RECIPE:");
        logRecipe(recipe);
        out_.println("PARAMETERS:");
        logParameters(params);
        out_.println("INPUT FRAMES:");
        logFrames(inFrames);
        FrameList outFrames = recipe.execute(params, inFrames, null);
        out_.println("OUTPUT FRAMES:");
        logFrames(outFrames);
        return outFrames;
    }

 
    public void logRecipe(Recipe recipe) {
        out_.println(recipe);
    }

    public void logParameter(Parameter param) {
        out_.println(param);
    }

    public void logFrame(Frame frame) {
        out_.println(frame);
    }

    public void logParameters(Parameter[] params) {
        for (int i = 0; i < params.length; i++) {
            logParameter(params[i]);
        }
    }

    public void logFrames(FrameList frames) {
        for (Iterator it = frames.iterator(); it.hasNext();) {
            logFrame((Frame) it.next());
        }
    }

    public void logFrames(Frame[] frames) {
        logFrames(new FrameList(frames));
    }

}
